@@ -26,8 +26,9 @@
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QPushButton>
+#include <QLineEdit>
+#include <QPointer>
 
-#include <KLineEdit>
 #include <KLocalizedString>
 
 class AssociatedFilesUI::Private
@@ -37,16 +38,16 @@ private:
 
 public:
     QLabel *labelGreeting;
-    KLineEdit *lineEditSourceUrl;
+    QLineEdit *lineEditSourceUrl;
     QRadioButton *radioNoCopyMove, *radioCopyFile, *radioMoveFile;
     QLabel *labelMoveCopyLocation;
-    KLineEdit *lineMoveCopyLocation;
+    QLineEdit *lineMoveCopyLocation;
     QGroupBox *groupBoxRename;
     QRadioButton *radioKeepFilename, *radioRenameToEntryId, *radioUserDefinedName;
-    KLineEdit *lineEditUserDefinedName;
+    QLineEdit *lineEditUserDefinedName;
     QGroupBox *groupBoxPathType;
     QRadioButton *radioRelativePath, *radioAbsolutePath;
-    KLineEdit *linePreview;
+    QLineEdit *linePreview;
 
     QUrl sourceUrl;
     QSharedPointer<Entry> entry;
@@ -65,7 +66,7 @@ public:
         layout->addWidget(labelGreeting);
         labelGreeting->setWordWrap(true);
 
-        lineEditSourceUrl = new KLineEdit(p);
+        lineEditSourceUrl = new QLineEdit(p);
         layout->addWidget(lineEditSourceUrl);
         lineEditSourceUrl->setReadOnly(true);
 
@@ -88,12 +89,16 @@ public:
         radioMoveFile = new QRadioButton(i18n("Move document next to bibliography file"), groupBox);
         groupBoxLayout->addWidget(radioMoveFile);
         buttonGroup->addButton(radioMoveFile);
+#if QT_VERSION >= 0x050f00
+        connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::idClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#else // QT_VERSION < 0x050f00
         connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#endif // QT_VERSION >= 0x050f00
         radioNoCopyMove->setChecked(true); /// by default
         groupBoxLayout->addSpacing(4);
         labelMoveCopyLocation = new QLabel(i18n("Path and filename of bibliography file:"), groupBox);
         groupBoxLayout->addWidget(labelMoveCopyLocation, 1);
-        lineMoveCopyLocation = new KLineEdit(groupBox);
+        lineMoveCopyLocation = new QLineEdit(groupBox);
         lineMoveCopyLocation->setReadOnly(true);
         groupBoxLayout->addWidget(lineMoveCopyLocation, 1);
 
@@ -113,10 +118,14 @@ public:
         radioUserDefinedName = new QRadioButton(i18n("User-defined name:"), groupBoxRename);
         gridLayout->addWidget(radioUserDefinedName, 2, 0, 1, 2);
         buttonGroup->addButton(radioUserDefinedName);
-        lineEditUserDefinedName = new KLineEdit(groupBoxRename);
+        lineEditUserDefinedName = new QLineEdit(groupBoxRename);
         gridLayout->addWidget(lineEditUserDefinedName, 3, 1, 1, 1);
-        connect(lineEditUserDefinedName, &KLineEdit::textEdited, p, &AssociatedFilesUI::updateUIandPreview);
+        connect(lineEditUserDefinedName, &QLineEdit::textEdited, p, &AssociatedFilesUI::updateUIandPreview);
+#if QT_VERSION >= 0x050f00
+        connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::idClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#else // QT_VERSION < 0x050f00
         connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#endif // QT_VERSION >= 0x050f00
         radioRenameToEntryId->setChecked(true); /// by default
 
         groupBoxPathType = new QGroupBox(i18n("Path as Inserted into Entry"), p);
@@ -129,7 +138,11 @@ public:
         radioAbsolutePath = new QRadioButton(i18n("Absolute Path"), groupBoxPathType);
         groupBoxLayout->addWidget(radioAbsolutePath);
         buttonGroup->addButton(radioAbsolutePath);
+#if QT_VERSION >= 0x050f00
+        connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::idClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#else // QT_VERSION < 0x050f00
         connect(buttonGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked), p, &AssociatedFilesUI::updateUIandPreview);
+#endif // QT_VERSION >= 0x050f00
         radioRelativePath->setChecked(true); /// by default
 
         layout->addSpacing(8);
@@ -137,7 +150,7 @@ public:
         label = new QLabel(i18n("Preview of reference to be inserted:"), p);
         layout->addWidget(label);
 
-        linePreview = new KLineEdit(p);
+        linePreview = new QLineEdit(p);
         layout->addWidget(linePreview);
         linePreview->setReadOnly(true);
 
@@ -145,7 +158,7 @@ public:
     }
 };
 
-bool AssociatedFilesUI::associateUrl(const QUrl &url, QSharedPointer<Entry> &entry, const File *bibTeXfile, QWidget *parent) {
+QString AssociatedFilesUI::associateUrl(const QUrl &url, QSharedPointer<Entry> &entry, const File *bibTeXfile, const bool doInsertUrl, QWidget *parent) {
     QPointer<QDialog> dlg = new QDialog(parent);
     QBoxLayout *layout = new QVBoxLayout(dlg);
     QPointer<AssociatedFilesUI> ui = new AssociatedFilesUI(entry->id(), bibTeXfile, dlg);
@@ -157,57 +170,25 @@ bool AssociatedFilesUI::associateUrl(const QUrl &url, QSharedPointer<Entry> &ent
     connect(buttonBox->button(QDialogButtonBox::Ok), &QPushButton::clicked, dlg.data(), &QDialog::accept);
     connect(buttonBox->button(QDialogButtonBox::Cancel), &QPushButton::clicked, dlg.data(), &QDialog::reject);
 
-    if (AssociatedFiles::urlIsLocal(url))
+    if (url.isLocalFile())
         ui->setupForLocalFile(url, entry->id());
     else
         ui->setupForRemoteUrl(url, entry->id());
 
     const bool accepted = dlg->exec() == QDialog::Accepted;
-    bool success = true;
-    if (accepted) {
-        const QUrl newUrl = AssociatedFiles::copyDocument(url, entry->id(), bibTeXfile, ui->renameOperation(), ui->moveCopyOperation(), dlg, ui->userDefinedFilename());
-        success &= !newUrl.isEmpty();
-        if (success) {
-            const QString referenceString = AssociatedFiles::associateDocumentURL(newUrl, entry, bibTeXfile, ui->pathType());
-            success &= !referenceString.isEmpty();
-        }
-    }
-
-    delete dlg;
-    return accepted && success;
-}
-
-QString AssociatedFilesUI::associateUrl(const QUrl &url, const QString &entryId, const File *bibTeXfile, QWidget *parent) {
-    QPointer<QDialog> dlg = new QDialog(parent);
-    QBoxLayout *layout = new QVBoxLayout(dlg);
-    QPointer<AssociatedFilesUI> ui = new AssociatedFilesUI(entryId, bibTeXfile, dlg);
-    layout->addWidget(ui);
-    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dlg);
-    layout->addWidget(buttonBox);
-    dlg->setLayout(layout);
-
-    connect(buttonBox->button(QDialogButtonBox::Ok), &QPushButton::clicked, dlg.data(), &QDialog::accept);
-    connect(buttonBox->button(QDialogButtonBox::Cancel), &QPushButton::clicked, dlg.data(), &QDialog::reject);
-
-    if (AssociatedFiles::urlIsLocal(url))
-        ui->setupForLocalFile(url, entryId);
-    else
-        ui->setupForRemoteUrl(url, entryId);
-
-    const bool accepted = dlg->exec() == QDialog::Accepted;
-    bool success = true;
+    bool success = false;
     QString referenceString;
     if (accepted) {
-        const QUrl newUrl = AssociatedFiles::copyDocument(url, entryId, bibTeXfile, ui->renameOperation(), ui->moveCopyOperation(), dlg, ui->userDefinedFilename());
-        success &= !newUrl.isEmpty();
+        const QUrl newUrl = AssociatedFiles::copyDocument(url, entry->id(), bibTeXfile, ui->renameOperation(), ui->moveCopyOperation(), dlg, ui->userDefinedFilename());
+        success = newUrl.isValid();
         if (success) {
-            referenceString = AssociatedFiles::associateDocumentURL(newUrl, bibTeXfile, ui->pathType());
+            referenceString = doInsertUrl ? AssociatedFiles::insertUrl(newUrl, entry, bibTeXfile, ui->pathType()) : AssociatedFiles::computeAssociateString(newUrl, bibTeXfile, ui->pathType());
             success &= !referenceString.isEmpty();
         }
     }
 
     delete dlg;
-    return accepted && success ? referenceString : QString();
+    return success ? referenceString : QString();
 }
 
 AssociatedFilesUI::AssociatedFilesUI(const QString &entryId, const File *bibTeXfile, QWidget *parent)
@@ -223,21 +204,21 @@ AssociatedFilesUI::~AssociatedFilesUI()
 
 AssociatedFiles::RenameOperation AssociatedFilesUI::renameOperation() const {
     if (d->radioRenameToEntryId->isChecked())
-        return AssociatedFiles::roEntryId;
+        return AssociatedFiles::RenameOperation::EntryId;
     else if (d->radioKeepFilename->isChecked() || d->lineEditUserDefinedName->text().isEmpty())
-        return AssociatedFiles::roKeepName;
+        return AssociatedFiles::RenameOperation::KeepName;
     else
-        return AssociatedFiles::roUserDefined;
+        return AssociatedFiles::RenameOperation::UserDefined;
 }
 
 AssociatedFiles::MoveCopyOperation AssociatedFilesUI::moveCopyOperation() const {
-    if (d->radioNoCopyMove->isChecked()) return AssociatedFiles::mcoNoCopyMove;
-    else if (d->radioMoveFile->isChecked()) return AssociatedFiles::mcoMove;
-    else return AssociatedFiles::mcoCopy;
+    if (d->radioNoCopyMove->isChecked()) return AssociatedFiles::MoveCopyOperation::None;
+    else if (d->radioMoveFile->isChecked()) return AssociatedFiles::MoveCopyOperation::Move;
+    else return AssociatedFiles::MoveCopyOperation::Copy;
 }
 
 AssociatedFiles::PathType AssociatedFilesUI::pathType() const {
-    return d->radioAbsolutePath->isChecked() ? AssociatedFiles::ptAbsolute : AssociatedFiles::ptRelative;
+    return d->radioAbsolutePath->isChecked() ? AssociatedFiles::PathType::Absolute : AssociatedFiles::PathType::Relative;
 }
 
 QString AssociatedFilesUI::userDefinedFilename() const {
@@ -251,30 +232,56 @@ void AssociatedFilesUI::updateUIandPreview() {
     QString preview = i18n("No preview available");
     const QString entryId = d->entryId.isEmpty() && !d->entry.isNull() ? d->entry->id() : d->entryId;
 
+    const QUrl bibTeXfileUrl = d->bibTeXfile != nullptr && d->bibTeXfile->hasProperty(File::Url) ? d->bibTeXfile->property(File::Url).toUrl() : QUrl();
+
     if (entryId.isEmpty()) {
+        /// If current entry has no identifier, then renaming after entry id is not possible
         d->radioRenameToEntryId->setEnabled(false);
+        /// ... and the current filename should be kept
         d->radioKeepFilename->setChecked(true);
     } else
+        /// But if the entry has an identifier, prefer to rename after it
         d->radioRenameToEntryId->setEnabled(true);
-    if (d->bibTeXfile == nullptr || !d->bibTeXfile->hasProperty(File::Url)) {
+    if (!bibTeXfileUrl.isValid()) {
+        /// If current file has no URL, e.g. because it hasn't been saved,
+        /// no relative path can be assembled, i.e. only an absolute one is possible
+        d->radioRelativePath->setEnabled(false);
+        d->radioAbsolutePath->setChecked(true);
+    } else if (bibTeXfileUrl.isValid() && !d->sourceUrl.isRelative() && (bibTeXfileUrl.scheme() != d->sourceUrl.scheme() || (bibTeXfileUrl.scheme() == d->sourceUrl.scheme() && bibTeXfileUrl.host() != d->sourceUrl.host()))) {
+        /// If URL to be associated is not relative, i.e. has a scheme and this scheme is
+        /// different from the current bibliography file's scheme or the scheme is the same
+        /// but the host is different, then no relative path is possible
         d->radioRelativePath->setEnabled(false);
         d->radioAbsolutePath->setChecked(true);
-        d->labelMoveCopyLocation->hide();
-        d->lineMoveCopyLocation->hide();
     } else {
+        /// If the file has a valid URL, prefer to use relative paths for association
         d->radioRelativePath->setEnabled(true);
+    }
+    if (bibTeXfileUrl.isValid()) {
+        /// Show the URL of the current bibliography file
+        d->lineMoveCopyLocation->setText(bibTeXfileUrl.path());
+        d->lineMoveCopyLocation->setToolTip(d->lineMoveCopyLocation->text());
         d->labelMoveCopyLocation->show();
         d->lineMoveCopyLocation->show();
-        d->lineMoveCopyLocation->setText(d->bibTeXfile->property(File::Url).toUrl().path());
+    } else {
+        d->labelMoveCopyLocation->hide();
+        d->lineMoveCopyLocation->hide();
     }
+    /// Renaming is only possible if remote file is either copied or moved
     d->groupBoxRename->setEnabled(!d->radioNoCopyMove->isChecked());
     if (d->radioNoCopyMove->isChecked())
+        /// Not moving/copying remote file implies that filename is kept
         d->radioKeepFilename->setChecked(true);
 
-    if (d->bibTeXfile != nullptr && !d->sourceUrl.isEmpty() && !entryId.isEmpty()) {
-        const QUrl newUrl = AssociatedFiles::copyDocument(d->sourceUrl, entryId, d->bibTeXfile, renameOperation(), moveCopyOperation(), nullptr, d->lineEditUserDefinedName->text(), true);
-        if (!newUrl.isEmpty())
-            preview = AssociatedFiles::associateDocumentURL(newUrl,  d->bibTeXfile, pathType());
+    if (d->radioMoveFile->isChecked() || d->radioCopyFile->isChecked()) {
+        /// Assuming that the remote URL is to be copied next to the bibliography file,
+        /// compute the destination a.k.a. target URL of the copied file
+        const QPair<QUrl, QUrl> newURLs = AssociatedFiles::computeSourceDestinationUrls(d->sourceUrl, entryId, d->bibTeXfile, renameOperation(), d->lineEditUserDefinedName->text());
+        if (newURLs.second.isValid())
+            preview = AssociatedFiles::computeAssociateString(newURLs.second, d->bibTeXfile, pathType());
+    } else if (d->radioNoCopyMove->isChecked()) {
+        if (d->sourceUrl.isValid())
+            preview = AssociatedFiles::computeAssociateString(d->sourceUrl, d->bibTeXfile, pathType());
     }
     d->linePreview->setText(preview);
 }
