@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2020 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -20,11 +20,12 @@
 #include <QLayout>
 #include <QBuffer>
 #include <QLabel>
+#include <QLineEdit>
+#include <QComboBox>
 #include <QTreeWidget>
 #include <QFileInfo>
 #include <QDropEvent>
 #include <QMenu>
-#include <QMimeDatabase>
 #include <QMimeType>
 #include <QMimeData>
 #include <QSortFilterProxyModel>
@@ -32,31 +33,38 @@
 #include <QPushButton>
 #include <QFontDatabase>
 #include <QRegularExpression>
+#include <QTimer>
 
+#include <kio_version.h>
 #include <KLocalizedString>
-#include <KLineEdit>
-#include <KComboBox>
+#if KIO_VERSION >= 0x054700 // >= 5.71.0
+#include <KIO/OpenUrlJob>
+#include <KIO/JobUiDelegate>
+#else // < 5.71.0
 #include <KRun>
+#endif // KIO_VERSION >= 0x054700
 #include <KTextEditor/Document>
 #include <KTextEditor/Editor>
 #include <KTextEditor/View>
+#include <KMessageWidget>
 #include <kio_version.h>
 
-#include "preferences.h"
-#include "idsuggestions.h"
-#include "fileinfo.h"
-#include "kbibtex.h"
-#include "bibtexentries.h"
-#include "bibtexfields.h"
-#include "fileimporterbibtex.h"
-#include "fileexporterbibtex.h"
-#include "file.h"
-#include "fieldinput.h"
-#include "entry.h"
-#include "macro.h"
-#include "preamble.h"
-#include "fieldlineedit.h"
-#include "delayedexecutiontimer.h"
+#include <KBibTeX>
+#include <Preferences>
+#include <BibTeXEntries>
+#include <BibTeXFields>
+#include <Entry>
+#include <Macro>
+#include <Preamble>
+#include <Comment>
+#include <File>
+#include <FileInfo>
+#include <FileImporterBibTeX>
+#include <FileExporterBibTeX>
+#include <FileExporterBibTeX2HTML>
+#include <IdSuggestions>
+#include "field/fieldinput.h"
+#include "field/fieldlineedit.h"
 #include "logging_gui.h"
 
 static const unsigned int interColumnSpace = 16;
@@ -89,7 +97,9 @@ void ElementWidget::gotModified()
 EntryConfiguredWidget::EntryConfiguredWidget(const QSharedPointer<const EntryTabLayout> &entryTabLayout, QWidget *parent)
         : ElementWidget(parent), fieldInputCount(entryTabLayout->singleFieldLayouts.size()), numCols(entryTabLayout->columns), etl(entryTabLayout)
 {
-    gridLayout = new QGridLayout(this);
+    vboxLayout = new QVBoxLayout(this);
+    gridLayout = new QGridLayout();
+    vboxLayout->addLayout(gridLayout, 100);
 
     /// Initialize list of field input widgets plus labels
     listOfLabeledFieldInput = new LabeledFieldInput*[fieldInputCount];
@@ -99,6 +109,9 @@ EntryConfiguredWidget::EntryConfiguredWidget(const QSharedPointer<const EntryTab
 
 EntryConfiguredWidget::~EntryConfiguredWidget()
 {
+    for (int i = fieldInputCount - 1; i >= 0; --i)
+        if (listOfLabeledFieldInput[i] != nullptr)
+            delete listOfLabeledFieldInput[i];
     delete[] listOfLabeledFieldInput;
 }
 
@@ -157,12 +170,17 @@ void EntryConfiguredWidget::showReqOptWidgets(bool forceVisible, const QString &
     layoutGUI(forceVisible, entryType);
 }
 
-void EntryConfiguredWidget::setReadOnly(bool isReadOnly)
+QString EntryConfiguredWidget::identifier() const
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    return etl->identifier;
+}
+
+void EntryConfiguredWidget::setReadOnly(bool _isReadOnly)
+{
+    ElementWidget::setReadOnly(_isReadOnly);
 
     for (QMap<QString, FieldInput *>::Iterator it = bibtexKeyToWidget.begin(); it != bibtexKeyToWidget.end(); ++it)
-        it.value()->setReadOnly(isReadOnly);
+        it.value()->setReadOnly(_isReadOnly);
 }
 
 QString EntryConfiguredWidget::label()
@@ -184,9 +202,9 @@ void EntryConfiguredWidget::setFile(const File *file)
             QStringList list = file->uniqueEntryValuesList(it.key());
             /// for crossref fields, add all entries' ids
             if (it.key().toLower() == Entry::ftCrossRef)
-                list.append(file->allKeys(File::etEntry));
+                list.append(file->allKeys(File::ElementType::Entry));
             /// add macro keys
-            list.append(file->allKeys(File::etMacro));
+            list.append(file->allKeys(File::ElementType::Macro));
 
             it.value()->setCompletionItems(list);
         }
@@ -200,6 +218,14 @@ bool EntryConfiguredWidget::canEdit(const Element *element)
     return Entry::isEntry(*element);
 }
 
+void EntryConfiguredWidget::infoMessageLinkActivated(const QString &contents)
+{
+    if (contents.startsWith(QStringLiteral("#tab:"))) {
+        const QString tabIdentifier = contents.mid(5);
+        emit requestingTabChange(tabIdentifier);
+    }
+}
+
 void EntryConfiguredWidget::createGUI()
 {
     int i = 0;
@@ -214,7 +240,7 @@ void EntryConfiguredWidget::createGUI()
         connect(labeledFieldInput->fieldInput, &FieldInput::modified, this, &EntryConfiguredWidget::gotModified);
 
         /// memorize if field input should grow vertically (e.g. is a list)
-        labeledFieldInput->isVerticallyMinimumExpaning = sfl.fieldInputLayout == KBibTeX::MultiLine || sfl.fieldInputLayout == KBibTeX::List || sfl.fieldInputLayout == KBibTeX::PersonList || sfl.fieldInputLayout == KBibTeX::KeywordList;
+        labeledFieldInput->isVerticallyMinimumExpaning = sfl.fieldInputLayout == KBibTeX::FieldInputType::MultiLine || sfl.fieldInputLayout == KBibTeX::FieldInputType::List || sfl.fieldInputLayout == KBibTeX::FieldInputType::PersonList || sfl.fieldInputLayout == KBibTeX::FieldInputType::KeywordList;
 
         /// create a label next to the editing widget
         labeledFieldInput->label = new QLabel(QString(QStringLiteral("%1:")).arg(sfl.uiLabel), this);
@@ -228,6 +254,13 @@ void EntryConfiguredWidget::createGUI()
         ++i;
     }
 
+    if (!etl->infoMessages.isEmpty())
+        for (const QString &infoMessage : etl->infoMessages) {
+            KMessageWidget *infoMessagesWidget = new KMessageWidget(i18n(infoMessage.toUtf8().constData()), this);
+            connect(infoMessagesWidget, &KMessageWidget::linkActivated, this, &EntryConfiguredWidget::infoMessageLinkActivated);
+            vboxLayout->addWidget(infoMessagesWidget, 1);
+        }
+
     layoutGUI(true);
 }
 
@@ -354,15 +387,15 @@ bool ReferenceWidget::reset(QSharedPointer<const Element> element)
 {
     /// if signals are not deactivated, the "modified" signal would be emitted when
     /// resetting the widgets' values
-    disconnect(entryType->lineEdit(), &KLineEdit::textChanged, this, &ReferenceWidget::gotModified);
-    disconnect(entryId, &KLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
+    const QSignalBlocker blockerEntryTypeLineEdit(entryType->lineEdit());
+    const QSignalBlocker blockerEntryIdLineEdit(entryId);
 
     bool result = false;
     QSharedPointer<const Entry> entry = element.dynamicCast<const Entry>();
     if (!entry.isNull()) {
         entryType->setEnabled(!isReadOnly);
         buttonSuggestId->setEnabled(!isReadOnly);
-        QString type = BibTeXEntries::instance().format(entry->type(), KBibTeX::cUpperCamelCase);
+        QString type = BibTeXEntries::instance().format(entry->type(), KBibTeX::Casing::UpperCamelCase);
         int index = entryType->findData(type);
         if (index == -1) {
             const QString typeLower(type.toLower());
@@ -397,9 +430,6 @@ bool ReferenceWidget::reset(QSharedPointer<const Element> element)
         }
     }
 
-    connect(entryId, &KLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
-    connect(entryType->lineEdit(), &KLineEdit::textChanged, this, &ReferenceWidget::gotModified);
-
     return result;
 }
 
@@ -428,12 +458,12 @@ bool ReferenceWidget::validate(QWidget **widgetWithIssue, QString &message) cons
     return true;
 }
 
-void ReferenceWidget::setReadOnly(bool isReadOnly)
+void ReferenceWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
 
-    entryId->setReadOnly(isReadOnly);
-    entryType->setEnabled(!isReadOnly);
+    entryId->setReadOnly(_isReadOnly);
+    entryType->setEnabled(!_isReadOnly);
 }
 
 QString ReferenceWidget::label()
@@ -470,7 +500,7 @@ void ReferenceWidget::createGUI()
 {
     QHBoxLayout *layout = new QHBoxLayout(this);
 
-    entryType = new KComboBox(this);
+    entryType = new QComboBox(this);
     entryType->setEditable(true);
     entryType->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
     QLabel *label = new QLabel(i18n("Type:"), this);
@@ -480,7 +510,7 @@ void ReferenceWidget::createGUI()
 
     layout->addSpacing(interColumnSpace);
 
-    entryId = new KLineEdit(this);
+    entryId = new QLineEdit(this);
     entryId->setClearButtonEnabled(true);
     label = new QLabel(i18n("Id:"), this);
     label->setBuddy(entryId);
@@ -504,9 +534,9 @@ void ReferenceWidget::createGUI()
     QMenu *suggestionsMenu = new QMenu(buttonSuggestId);
     buttonSuggestId->setMenu(suggestionsMenu);
 
-    connect(entryType->lineEdit(), &KLineEdit::textChanged, this, &ReferenceWidget::gotModified);
-    connect(entryId, &KLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
-    connect(entryType->lineEdit(), &KLineEdit::textChanged, this, &ReferenceWidget::entryTypeChanged);
+    connect(entryType->lineEdit(), &QLineEdit::textChanged, this, &ReferenceWidget::gotModified);
+    connect(entryId, &QLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
+    connect(entryType->lineEdit(), &QLineEdit::textChanged, this, &ReferenceWidget::entryTypeChanged);
     connect(suggestionsMenu, &QMenu::aboutToShow, this, &ReferenceWidget::prepareSuggestionsMenu);
 }
 
@@ -592,10 +622,9 @@ void ReferenceWidget::setEntryIdByDefault()
         const QString defaultSuggestion = idSuggestions->defaultFormatId(*crossrefResolvedEntry.data());
 
         if (!defaultSuggestion.isEmpty()) {
-            disconnect(entryId, &KLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
+            const QSignalBlocker blocker(entryId);
             /// Apply default suggestion to widget
             entryId->setText(defaultSuggestion);
-            connect(entryId, &KLineEdit::textEdited, this, &ReferenceWidget::entryIdManuallyChanged);
         }
     }
 }
@@ -603,7 +632,7 @@ void ReferenceWidget::setEntryIdByDefault()
 QString ReferenceWidget::computeType() const
 {
     if (entryType->currentIndex() < 0 || entryType->lineEdit()->isModified())
-        return BibTeXEntries::instance().format(entryType->lineEdit()->text(), KBibTeX::cUpperCamelCase);
+        return BibTeXEntries::instance().format(entryType->lineEdit()->text(), KBibTeX::Casing::UpperCamelCase);
     else
         return entryType->itemData(entryType->currentIndex()).toString();
 }
@@ -612,7 +641,7 @@ FilesWidget::FilesWidget(QWidget *parent)
         : ElementWidget(parent)
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
-    fileList = new FieldInput(KBibTeX::UrlList, KBibTeX::tfVerbatim /* eventually ignored, see constructor of UrlListEdit */, KBibTeX::tfVerbatim /* eventually ignored, see constructor of UrlListEdit */, this);
+    fileList = new FieldInput(KBibTeX::FieldInputType::UrlList, KBibTeX::TypeFlag::Verbatim /* eventually ignored, see constructor of UrlListEdit */, KBibTeX::TypeFlag::Verbatim /* eventually ignored, see constructor of UrlListEdit */, this);
     fileList->setFieldKey(QStringLiteral("^external"));
     layout->addWidget(fileList);
     connect(fileList, &FieldInput::modified, this, &FilesWidget::gotModified);
@@ -680,10 +709,10 @@ bool FilesWidget::apply(QSharedPointer<Element> element) const
     if (localFileValue.isEmpty()) {
         entry->remove(Entry::ftFile);
         entry->remove(Entry::ftLocalFile);
-    } else if (Preferences::instance().bibliographySystem() == Preferences::BibLaTeX) {
+    } else if (Preferences::instance().bibliographySystem() == Preferences::BibliographySystem::BibLaTeX) {
         entry->remove(Entry::ftLocalFile);
         entry->insert(Entry::ftFile, localFileValue);
-    } else if (Preferences::instance().bibliographySystem() == Preferences::BibTeX) {
+    } else if (Preferences::instance().bibliographySystem() == Preferences::BibliographySystem::BibTeX) {
         entry->remove(Entry::ftFile);
         entry->insert(Entry::ftLocalFile, localFileValue);
     }
@@ -720,10 +749,10 @@ bool FilesWidget::validate(QWidget **widgetWithIssue, QString &message) const
     return fileList->validate(widgetWithIssue, message);
 }
 
-void FilesWidget::setReadOnly(bool isReadOnly)
+void FilesWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
-    fileList->setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
+    fileList->setReadOnly(_isReadOnly);
 }
 
 QString FilesWidget::label()
@@ -800,12 +829,12 @@ bool OtherFieldsWidget::validate(QWidget **, QString &) const
     return true;
 }
 
-void OtherFieldsWidget::setReadOnly(bool isReadOnly)
+void OtherFieldsWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
 
-    fieldName->setReadOnly(isReadOnly);
-    fieldContent->setReadOnly(isReadOnly);
+    fieldName->setReadOnly(_isReadOnly);
+    fieldContent->setReadOnly(_isReadOnly);
 
     /// will take care of enabled/disabling buttons
     updateGUI();
@@ -903,11 +932,13 @@ void OtherFieldsWidget::actionOpen()
         QMimeType mimeType = FileInfo::mimeTypeForUrl(currentUrl);
         const QString mimeTypeName = mimeType.name();
         /// Ask KDE subsystem to open url in viewer matching mime type
-#if KIO_VERSION < 0x051f00 // < 5.31.0
-        KRun::runUrl(currentUrl, mimeTypeName, this, false, false);
-#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+#if KIO_VERSION < 0x054700 // < 5.71.0
         KRun::runUrl(currentUrl, mimeTypeName, this, KRun::RunFlags());
-#endif // KIO_VERSION < 0x051f00
+#else // KIO_VERSION < 0x054700 // >= 5.71.0
+        KIO::OpenUrlJob *job = new KIO::OpenUrlJob(currentUrl, mimeTypeName);
+        job->setUiDelegate(new KIO::JobUiDelegate());
+        job->start();
+#endif // KIO_VERSION < 0x054700
     }
 }
 
@@ -926,9 +957,9 @@ void OtherFieldsWidget::createGUI()
 
     QLabel *label = new QLabel(i18n("Name:"), this);
     layout->addWidget(label, 0, 0, 1, 1);
-    label->setAlignment((Qt::Alignment)label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment));
+    label->setAlignment(static_cast<Qt::Alignment>(label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment)));
 
-    fieldName = new KLineEdit(this);
+    fieldName = new QLineEdit(this);
     layout->addWidget(fieldName, 0, 1, 1, 1);
     label->setBuddy(fieldName);
 
@@ -938,14 +969,14 @@ void OtherFieldsWidget::createGUI()
 
     label = new QLabel(i18n("Content:"), this);
     layout->addWidget(label, 1, 0, 1, 1);
-    label->setAlignment((Qt::Alignment)label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment));
-    fieldContent = new FieldInput(KBibTeX::MultiLine, KBibTeX::tfSource, KBibTeX::tfSource, this);
+    label->setAlignment(static_cast<Qt::Alignment>(label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment)));
+    fieldContent = new FieldInput(KBibTeX::FieldInputType::MultiLine, KBibTeX::TypeFlag::Source, KBibTeX::TypeFlag::Source, this);
     layout->addWidget(fieldContent, 1, 1, 1, 2);
     label->setBuddy(fieldContent->buddy());
 
     label = new QLabel(i18n("List:"), this);
     layout->addWidget(label, 2,  0, 1, 1);
-    label->setAlignment((Qt::Alignment)label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment));
+    label->setAlignment(static_cast<Qt::Alignment>(label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment)));
 
     otherFieldsList = new QTreeWidget(this);
     otherFieldsList->setHeaderLabels(QStringList {i18n("Key"), i18n("Value")});
@@ -963,7 +994,7 @@ void OtherFieldsWidget::createGUI()
     connect(otherFieldsList, &QTreeWidget::itemActivated, this, &OtherFieldsWidget::listElementExecuted);
     connect(otherFieldsList, &QTreeWidget::currentItemChanged, this, &OtherFieldsWidget::listCurrentChanged);
     connect(otherFieldsList, &QTreeWidget::itemSelectionChanged, this, &OtherFieldsWidget::updateGUI);
-    connect(fieldName, &KLineEdit::textEdited, this, &OtherFieldsWidget::updateGUI);
+    connect(fieldName, &QLineEdit::textEdited, this, &OtherFieldsWidget::updateGUI);
     connect(buttonAddApply, &QPushButton::clicked, this, &OtherFieldsWidget::actionAddApply);
     connect(buttonDelete, &QPushButton::clicked, this, &OtherFieldsWidget::actionDelete);
     connect(buttonOpen, &QPushButton::clicked, this, &OtherFieldsWidget::actionOpen);
@@ -1038,11 +1069,11 @@ bool MacroWidget::validate(QWidget **widgetWithIssue, QString &message) const
     return fieldInputValue->validate(widgetWithIssue, message);
 }
 
-void MacroWidget::setReadOnly(bool isReadOnly)
+void MacroWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
 
-    fieldInputValue->setReadOnly(isReadOnly);
+    fieldInputValue->setReadOnly(_isReadOnly);
 }
 
 QString MacroWidget::label()
@@ -1066,8 +1097,8 @@ void MacroWidget::createGUI()
 
     QLabel *label = new QLabel(i18n("Value:"), this);
     layout->addWidget(label, 0);
-    label->setAlignment((Qt::Alignment)label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment));
-    fieldInputValue = new FieldInput(KBibTeX::MultiLine, KBibTeX::tfPlainText, KBibTeX::tfPlainText | KBibTeX::tfSource, this);
+    label->setAlignment(static_cast<Qt::Alignment>(label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment)));
+    fieldInputValue = new FieldInput(KBibTeX::FieldInputType::MultiLine, KBibTeX::TypeFlag::PlainText, KBibTeX::TypeFlag::PlainText | KBibTeX::TypeFlag::Source, this);
     layout->addWidget(fieldInputValue, 1);
     label->setBuddy(fieldInputValue->buddy());
 
@@ -1108,11 +1139,11 @@ bool PreambleWidget::validate(QWidget **widgetWithIssue, QString &message) const
     return fieldInputValue->validate(widgetWithIssue, message);
 }
 
-void PreambleWidget::setReadOnly(bool isReadOnly)
+void PreambleWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
 
-    fieldInputValue->setReadOnly(isReadOnly);
+    fieldInputValue->setReadOnly(_isReadOnly);
 }
 
 QString PreambleWidget::label()
@@ -1136,8 +1167,8 @@ void PreambleWidget::createGUI()
 
     QLabel *label = new QLabel(i18n("Value:"), this);
     layout->addWidget(label, 0);
-    label->setAlignment((Qt::Alignment)label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment));
-    fieldInputValue = new FieldInput(KBibTeX::MultiLine, KBibTeX::tfSource, KBibTeX::tfSource, this); // FIXME: other editing modes beyond Source applicable?
+    label->setAlignment(static_cast<Qt::Alignment>(label->style()->styleHint(QStyle::SH_FormLayoutLabelAlignment)));
+    fieldInputValue = new FieldInput(KBibTeX::FieldInputType::MultiLine, KBibTeX::TypeFlag::Source, KBibTeX::TypeFlag::Source, this); // FIXME: other editing modes beyond Source applicable?
     layout->addWidget(fieldInputValue, 1);
     label->setBuddy(fieldInputValue->buddy());
 
@@ -1148,31 +1179,34 @@ void PreambleWidget::createGUI()
 class SourceWidget::Private
 {
 public:
-    KComboBox *messages;
+    QComboBox *messages;
     QPushButton *buttonRestore;
     FileImporterBibTeX *importerBibTeX;
-    DelayedExecutionTimer *delayedExecutionTimer;
+    QTimer *delayedTimer;
 
     Private(SourceWidget *parent)
-            : messages(nullptr), buttonRestore(nullptr), importerBibTeX(new FileImporterBibTeX(parent)), delayedExecutionTimer(new DelayedExecutionTimer(1500, 500, parent)) {
-        /// nothing
+            : messages(nullptr), buttonRestore(nullptr), importerBibTeX(new FileImporterBibTeX(parent)), delayedTimer(new QTimer(parent)) {
+        delayedTimer->setSingleShot(true);
+        importerBibTeX->setCommentHandling(FileImporterBibTeX::CommentHandling::Keep);
     }
 
     void addMessage(const FileImporter::MessageSeverity severity, const QString &messageText)
     {
-        const QIcon icon = severity == FileImporter::SeverityInfo ? QIcon::fromTheme(QStringLiteral("dialog-information")) : (severity == FileImporter::SeverityWarning ? QIcon::fromTheme(QStringLiteral("dialog-warning")) : (severity == FileImporter::SeverityError ? QIcon::fromTheme(QStringLiteral("dialog-error")) : QIcon::fromTheme(QStringLiteral("dialog-question"))));
+        const QIcon icon = severity == FileImporter::MessageSeverity::Info ? QIcon::fromTheme(QStringLiteral("dialog-information")) : (severity == FileImporter::MessageSeverity::Warning ? QIcon::fromTheme(QStringLiteral("dialog-warning")) : (severity == FileImporter::MessageSeverity::Error ? QIcon::fromTheme(QStringLiteral("dialog-error")) : QIcon::fromTheme(QStringLiteral("dialog-question"))));
         messages->addItem(icon, messageText);
     }
 };
 
 SourceWidget::SourceWidget(QWidget *parent)
-        : ElementWidget(parent), elementClass(elementInvalid), d(new SourceWidget::Private(this))
+        : ElementWidget(parent), elementClass(ElementClass::Invalid), d(new SourceWidget::Private(this))
 {
     createGUI();
 
-    connect(document, &KTextEditor::Document::textChanged, d->delayedExecutionTimer, &DelayedExecutionTimer::trigger);
-    connect(document, &KTextEditor::Document::textChanged, d->messages, &KComboBox::clear);
-    connect(d->delayedExecutionTimer, &DelayedExecutionTimer::triggered, this, &SourceWidget::updateMessage);
+    connect(document, &KTextEditor::Document::textChanged, this, [this]() {
+        d->delayedTimer->start(500);
+    });
+    connect(document, &KTextEditor::Document::textChanged, d->messages, &QComboBox::clear);
+    connect(d->delayedTimer, &QTimer::timeout, this, &SourceWidget::updateMessage);
 }
 
 SourceWidget::~SourceWidget()
@@ -1198,26 +1232,34 @@ bool SourceWidget::apply(QSharedPointer<Element> element) const
     QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
     QSharedPointer<Entry> readEntry = file->first().dynamicCast<Entry>();
     if (!readEntry.isNull() && !entry.isNull()) {
-        if (elementClass != elementEntry) return false; ///< Source widget should only edit Entry objects
+        if (elementClass != ElementClass::Entry) return false; ///< Source widget should only edit Entry objects
         entry->operator =(*readEntry.data()); //entry = readEntry;
         return true;
     } else {
         QSharedPointer<Macro> macro = element.dynamicCast<Macro>();
         QSharedPointer<Macro> readMacro = file->first().dynamicCast<Macro>();
         if (!readMacro.isNull() && !macro.isNull()) {
-            if (elementClass != elementMacro) return false; ///< Source widget should only edit Macro objects
+            if (elementClass != ElementClass::Macro) return false; ///< Source widget should only edit Macro objects
             macro->operator =(*readMacro.data());
             return true;
         } else {
             QSharedPointer<Preamble> preamble = element.dynamicCast<Preamble>();
             QSharedPointer<Preamble> readPreamble = file->first().dynamicCast<Preamble>();
             if (!readPreamble.isNull() && !preamble.isNull()) {
-                if (elementClass != elementPreamble) return false; ///< Source widget should only edit Preamble objects
+                if (elementClass != ElementClass::Preamble) return false; ///< Source widget should only edit Preamble objects
                 preamble->operator =(*readPreamble.data());
                 return true;
             } else {
-                qCWarning(LOG_KBIBTEX_GUI) << "Do not know how to apply source code";
-                return false;
+                QSharedPointer<Comment> comment = element.dynamicCast<Comment>();
+                QSharedPointer<Comment> readComment = file->first().dynamicCast<Comment>();
+                if (!readComment.isNull() && !comment.isNull()) {
+                    if (elementClass != ElementClass::Comment) return false; ///< Source widget should only edit Comment objects
+                    comment->operator =(*readComment.data());
+                    return true;
+                } else {
+                    qCWarning(LOG_KBIBTEX_GUI) << "Do not know how to apply source code";
+                    return false;
+                }
             }
         }
     }
@@ -1227,7 +1269,7 @@ bool SourceWidget::reset(QSharedPointer<const Element> element)
 {
     /// if signals are not deactivated, the "modified" signal would be emitted when
     /// resetting the widget's value
-    disconnect(document, &KTextEditor::Document::textChanged, this, &SourceWidget::gotModified);
+    const QSignalBlocker blocker(document);
 
     FileExporterBibTeX exporter(this);
     exporter.setEncoding(QStringLiteral("utf-8"));
@@ -1245,8 +1287,6 @@ bool SourceWidget::reset(QSharedPointer<const Element> element)
     } else
         qCWarning(LOG_KBIBTEX_GUI) << "Converting entry to BibTeX source resulting in empty text";
 
-    connect(document, &KTextEditor::Document::textChanged, this, &SourceWidget::gotModified);
-
     return !exportedText.isEmpty();
 }
 
@@ -1267,45 +1307,51 @@ bool SourceWidget::validate(QWidget **widgetWithIssue, QString &message) const
 
     bool result = false;
     switch (elementClass) {
-    case elementEntry: {
+    case ElementClass::Entry: {
         QSharedPointer<Entry> entry = file->first().dynamicCast<Entry>();
         result = !entry.isNull();
         if (!result) message = i18n("Given source code does not parse as one single BibTeX entry.");
     }
     break;
-    case elementMacro: {
+    case ElementClass::Macro: {
         QSharedPointer<Macro> macro = file->first().dynamicCast<Macro>();
         result = !macro.isNull();
         if (!result) message = i18n("Given source code does not parse as one single BibTeX macro.");
     }
     break;
-    case elementPreamble: {
+    case ElementClass::Preamble: {
         QSharedPointer<Preamble> preamble = file->first().dynamicCast<Preamble>();
         result = !preamble.isNull();
         if (!result) message = i18n("Given source code does not parse as one single BibTeX preamble.");
     }
     break;
-    // case elementComment // TODO?
-    default:
-        message = QString(QStringLiteral("elementClass is unknown: %1")).arg(elementClass);
+    case ElementClass::Comment: {
+        QSharedPointer<Comment> comment = file->first().dynamicCast<Comment>();
+        result = !comment.isNull();
+        if (!result) message = i18n("Given source code does not parse as one single BibTeX comment.");
+    }
+    break;
+    case ElementClass::Invalid: {
+        message = i18n("Element class not properly set");
         result = false;
     }
+    }
 
     if (!result && widgetWithIssue != nullptr)
         *widgetWithIssue = document->views().at(0); ///< We create one view initially, so this should never fail
 
     if (message.isEmpty() && d->messages->count() == 0)
-        d->addMessage(FileImporter::SeverityInfo, i18n("No issues detected"));
+        d->addMessage(FileImporter::MessageSeverity::Info, i18n("No issues detected"));
 
     return result;
 }
 
-void SourceWidget::setReadOnly(bool isReadOnly)
+void SourceWidget::setReadOnly(bool _isReadOnly)
 {
-    ElementWidget::setReadOnly(isReadOnly);
+    ElementWidget::setReadOnly(_isReadOnly);
 
-    d->buttonRestore->setEnabled(!isReadOnly);
-    document->setReadWrite(!isReadOnly);
+    d->buttonRestore->setEnabled(!_isReadOnly);
+    document->setReadWrite(!_isReadOnly);
 }
 
 QString SourceWidget::label()
@@ -1338,7 +1384,7 @@ void SourceWidget::createGUI()
     KTextEditor::View *view = document->createView(this);
     layout->addWidget(view, 0, 0, 1, 2);
 
-    d->messages = new KComboBox(this);
+    d->messages = new QComboBox(this);
     layout->addWidget(d->messages, 1, 0, 1, 1);
 
     d->buttonRestore = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-undo")), i18n("Restore"), this);
@@ -1351,12 +1397,10 @@ void SourceWidget::reset()
 {
     /// if signals are not deactivated, the "modified" signal would be emitted when
     /// resetting the widget's value
-    disconnect(document, &KTextEditor::Document::textChanged, this, &SourceWidget::gotModified);
+    const QSignalBlocker blocker(document);
 
     document->setText(originalText);
     setModified(false);
-
-    connect(document, &KTextEditor::Document::textChanged, this, &SourceWidget::gotModified);
 }
 
 void SourceWidget::addMessage(const FileImporter::MessageSeverity severity, const QString &messageText)
@@ -1371,10 +1415,8 @@ void SourceWidget::updateMessage()
 
     if (!message.isEmpty()) {
         if (validationResult)
-            addMessage(FileImporter::SeverityInfo, message);
+            addMessage(FileImporter::MessageSeverity::Info, message);
         else
-            addMessage(FileImporter::SeverityError, message);
+            addMessage(FileImporter::MessageSeverity::Error, message);
     }
 }
-
-#include "elementwidgets.moc"
