@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2020 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,7 +23,6 @@
 #include <QClipboard>
 #include <QScrollArea>
 #include <QLayout>
-#include <QSignalMapper>
 #include <QCheckBox>
 #include <QDragEnterEvent>
 #include <QDropEvent>
@@ -42,14 +41,14 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 
-#include "fileinfo.h"
-#include "file.h"
-#include "entry.h"
-#include "fileimporterbibtex.h"
-#include "fileexporterbibtex.h"
+#include <File>
+#include <Entry>
+#include <FileImporterBibTeX>
+#include <FileExporterBibTeX>
+#include <FileInfo>
+#include <AssociatedFiles>
 #include "fieldlineedit.h"
-#include "associatedfiles.h"
-#include "associatedfilesui.h"
+#include "element/associatedfilesui.h"
 #include "logging_gui.h"
 
 class FieldListEdit::FieldListEditProtected
@@ -57,7 +56,6 @@ class FieldListEdit::FieldListEditProtected
 private:
     FieldListEdit *p;
     const int innerSpacing;
-    QSignalMapper *smRemove, *smGoUp, *smGoDown;
     QVBoxLayout *layout;
     KBibTeX::TypeFlag preferredTypeFlag;
     KBibTeX::TypeFlags typeFlags;
@@ -76,9 +74,6 @@ public:
 
     FieldListEditProtected(KBibTeX::TypeFlag ptf, KBibTeX::TypeFlags tf, FieldListEdit *parent)
             : p(parent), innerSpacing(4), preferredTypeFlag(ptf), typeFlags(tf), file(nullptr), m_isReadOnly(false) {
-        smRemove = new QSignalMapper(parent);
-        smGoUp = new QSignalMapper(parent);
-        smGoDown = new QSignalMapper(parent);
         setupGUI();
     }
 
@@ -112,10 +107,6 @@ public:
 
         layout->addStretch(100);
 
-        connect(smRemove, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineRemove);
-        connect(smGoDown, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineGoDown);
-        connect(smGoUp, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), p, &FieldListEdit::lineGoUp);
-
         scrollArea->setBackgroundRole(QPalette::Base);
         scrollArea->ensureWidgetVisible(container);
         scrollArea->setWidgetResizable(true);
@@ -150,20 +141,35 @@ public:
         QPushButton *remove = new QPushButton(QIcon::fromTheme(QStringLiteral("list-remove")), QString(), le);
         remove->setToolTip(i18n("Remove value"));
         le->appendWidget(remove);
-        connect(remove, &QPushButton::clicked, smRemove, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
-        smRemove->setMapping(remove, le);
+        connect(remove, &QPushButton::clicked, p, [this, le]() {
+            removeFieldLineEdit(le);
+            const QSize size(container->width(), recommendedHeight());
+            container->resize(size);
+            /// Instead of an 'emit' ...
+            QMetaObject::invokeMethod(p, "modified", Qt::DirectConnection, QGenericReturnArgument());
+        });
 
         QPushButton *goDown = new QPushButton(QIcon::fromTheme(QStringLiteral("go-down")), QString(), le);
         goDown->setToolTip(i18n("Move value down"));
         le->appendWidget(goDown);
-        connect(goDown, &QPushButton::clicked, smGoDown, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
-        smGoDown->setMapping(goDown, le);
+        connect(goDown, &QPushButton::clicked, p, [this, le]() {
+            const bool gotModified = goDownFieldLineEdit(le);
+            if (gotModified) {
+                /// Instead of an 'emit' ...
+                QMetaObject::invokeMethod(p, "modified", Qt::DirectConnection, QGenericReturnArgument());
+            }
+        });
 
         QPushButton *goUp = new QPushButton(QIcon::fromTheme(QStringLiteral("go-up")), QString(), le);
         goUp->setToolTip(i18n("Move value up"));
         le->appendWidget(goUp);
-        connect(goUp, &QPushButton::clicked, smGoUp, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
-        smGoUp->setMapping(goUp, le);
+        connect(goUp, &QPushButton::clicked, p, [this, le]() {
+            const bool gotModified = goUpFieldLineEdit(le);
+            if (gotModified) {
+                /// Instead of an 'emit' ...
+                QMetaObject::invokeMethod(p, "modified", Qt::DirectConnection, QGenericReturnArgument());
+            }
+        });
 
         connect(le, &FieldLineEdit::modified, p, &FieldListEdit::modified);
 
@@ -192,24 +198,28 @@ public:
         delete fieldLineEdit;
     }
 
-    void goDownFieldLineEdit(FieldLineEdit *fieldLineEdit) {
+    bool goDownFieldLineEdit(FieldLineEdit *fieldLineEdit) {
         int idx = lineEditList.indexOf(fieldLineEdit);
         if (idx < lineEditList.count() - 1) {
             layout->removeWidget(fieldLineEdit);
             lineEditList.removeOne(fieldLineEdit);
             lineEditList.insert(idx + 1, fieldLineEdit);
             layout->insertWidget(idx + 1, fieldLineEdit);
+            return true; ///< return 'true' upon actual modification
         }
+        return false; ///< return 'false' if nothing changed, i.e. FieldLineEdit is already at bottom
     }
 
-    void goUpFieldLineEdit(FieldLineEdit *fieldLineEdit) {
+    bool goUpFieldLineEdit(FieldLineEdit *fieldLineEdit) {
         int idx = lineEditList.indexOf(fieldLineEdit);
         if (idx > 0) {
             layout->removeWidget(fieldLineEdit);
             lineEditList.removeOne(fieldLineEdit);
             lineEditList.insert(idx - 1, fieldLineEdit);
             layout->insertWidget(idx - 1, fieldLineEdit);
+            return true; ///< return 'true' upon actual modification
         }
+        return false; ///< return 'false' if nothing changed, i.e. FieldLineEdit is already at top
     }
 };
 
@@ -324,18 +334,19 @@ void FieldListEdit::dragEnterEvent(QDragEnterEvent *event)
 
 void FieldListEdit::dropEvent(QDropEvent *event)
 {
-    const QString clipboardText = event->mimeData()->text();
+    const QString clipboardText = QString::fromUtf8(event->mimeData()->data(QStringLiteral("text/plain")));
+    event->acceptProposedAction();
     if (clipboardText.isEmpty()) return;
 
-    const File *file = nullptr;
+    bool success = false;
     if (!d->fieldKey.isEmpty() && clipboardText.startsWith(QStringLiteral("@"))) {
         FileImporterBibTeX importer(this);
-        file = importer.fromString(clipboardText);
-        const QSharedPointer<Entry> entry = (file != nullptr && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
+        QScopedPointer<const File> file(importer.fromString(clipboardText));
+        const QSharedPointer<Entry> entry = (!file.isNull() && file->count() == 1) ? file->first().dynamicCast<Entry>() : QSharedPointer<Entry>();
 
-        if (file != nullptr && !entry.isNull() && d->fieldKey == QStringLiteral("^external")) {
+        if (!entry.isNull() && d->fieldKey == QStringLiteral("^external")) {
             /// handle "external" list differently
-            const auto urlList = FileInfo::entryUrls(entry, QUrl(file->property(File::Url).toUrl()), FileInfo::TestExistenceNo);
+            const auto urlList = FileInfo::entryUrls(entry, QUrl(file->property(File::Url).toUrl()), FileInfo::TestExistence::No);
             Value v;
             v.reserve(urlList.size());
             for (const QUrl &url : urlList) {
@@ -343,17 +354,18 @@ void FieldListEdit::dropEvent(QDropEvent *event)
             }
             reset(v);
             emit modified();
-            return;
+            success = true;
         } else if (!entry.isNull() && entry->contains(d->fieldKey)) {
             /// case for "normal" lists like for authors, editors, ...
             reset(entry->value(d->fieldKey));
             emit modified();
-            return;
+            success = true;
         }
     }
 
-    if (file == nullptr || file->count() == 0) {
-        /// fall-back case: single field line edit with text
+    if (!success) {
+        /// In case above cases were not met and thus 'success' is still false,
+        /// clear this list edit and use the clipboad text as its single and only list element
         d->removeAllFieldLineEdits();
         FieldLineEdit *fle = addFieldLineEdit();
         fle->setText(clipboardText);
@@ -378,29 +390,6 @@ void FieldListEdit::lineAdd()
     newEdit->setFocus(Qt::ShortcutFocusReason);
 }
 
-void FieldListEdit::lineRemove(QWidget *widget)
-{
-    FieldLineEdit *fieldLineEdit = static_cast<FieldLineEdit *>(widget);
-    d->removeFieldLineEdit(fieldLineEdit);
-    QSize size(d->container->width(), d->recommendedHeight());
-    d->container->resize(size);
-    emit modified();
-}
-
-void FieldListEdit::lineGoDown(QWidget *widget)
-{
-    FieldLineEdit *fieldLineEdit = static_cast<FieldLineEdit *>(widget);
-    d->goDownFieldLineEdit(fieldLineEdit);
-    emit modified();
-}
-
-void FieldListEdit::lineGoUp(QWidget *widget)
-{
-    FieldLineEdit *fieldLineEdit = static_cast<FieldLineEdit *>(widget);
-    d->goUpFieldLineEdit(fieldLineEdit);
-    emit modified();
-}
-
 PersonListEdit::PersonListEdit(KBibTeX::TypeFlag preferredTypeFlag, KBibTeX::TypeFlags typeFlags, QWidget *parent)
         : FieldListEdit(preferredTypeFlag, typeFlags, parent)
 {
@@ -470,64 +459,60 @@ void PersonListEdit::slotAddNamesFromClipboard()
 
 
 UrlListEdit::UrlListEdit(QWidget *parent)
-        : FieldListEdit(KBibTeX::tfVerbatim, KBibTeX::tfVerbatim, parent)
+        : FieldListEdit(KBibTeX::TypeFlag::Verbatim, KBibTeX::TypeFlag::Verbatim, parent)
 {
-    m_signalMapperSaveLocallyButtonClicked = new QSignalMapper(this);
-    connect(m_signalMapperSaveLocallyButtonClicked, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), this, &UrlListEdit::slotSaveLocally);
-    m_signalMapperFieldLineEditTextChanged = new QSignalMapper(this);
-    connect(m_signalMapperFieldLineEditTextChanged, static_cast<void(QSignalMapper::*)(QWidget *)>(&QSignalMapper::mapped), this, &UrlListEdit::textChanged);
-
     m_buttonAddFile = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add file..."), this);
     addButton(m_buttonAddFile);
     QMenu *menuAddFile = new QMenu(m_buttonAddFile);
     m_buttonAddFile->setMenu(menuAddFile);
     connect(m_buttonAddFile, &QPushButton::clicked, m_buttonAddFile, &QPushButton::showMenu);
 
-    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference ..."), this, SLOT(slotAddReference()));
-    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference from clipboard"), this, SLOT(slotAddReferenceFromClipboard()));
+    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference ..."), this, [this]() {
+        slotAddReference();
+    });
+    menuAddFile->addAction(QIcon::fromTheme(QStringLiteral("emblem-symbolic-link")), i18n("Add reference from clipboard"), this, [this]() {
+        slotAddReferenceFromClipboard();
+    });
 }
 
 void UrlListEdit::slotAddReference()
 {
     QUrl bibtexUrl(d->file != nullptr ? d->file->property(File::Url, QVariant()).toUrl() : QUrl());
-    if (!bibtexUrl.isEmpty()) {
+    if (bibtexUrl.isValid()) {
         const QFileInfo fi(bibtexUrl.path());
         bibtexUrl.setPath(fi.absolutePath());
     }
     const QUrl documentUrl = QFileDialog::getOpenFileUrl(this, i18n("File to Associate"), bibtexUrl);
-    if (!documentUrl.isEmpty())
+    if (documentUrl.isValid())
         addReference(documentUrl);
 }
 
 void UrlListEdit::slotAddReferenceFromClipboard()
 {
     const QUrl url = QUrl::fromUserInput(QApplication::clipboard()->text());
-    if (!url.isEmpty())
+    if (url.isValid())
         addReference(url);
 }
 
 void UrlListEdit::addReference(const QUrl &url) {
     const Entry *entry = dynamic_cast<const Entry *>(m_element);
-    const QString entryId = entry != nullptr ? entry->id() : QString();
-    const QString visibleFilename = AssociatedFilesUI::associateUrl(url, entryId, d->file, this);
-    if (!visibleFilename.isEmpty()) {
-        Value *value = new Value();
-        value->append(QSharedPointer<VerbatimText>(new VerbatimText(visibleFilename)));
-        lineAdd(value);
-        delete value;
-        emit modified();
+    if (entry != nullptr) {
+        QSharedPointer<Entry> fakeTempEntry(new Entry(entry->type(), entry->id()));
+        const QString visibleFilename = AssociatedFilesUI::associateUrl(url, fakeTempEntry, d->file, false, this);
+        if (!visibleFilename.isEmpty()) {
+            Value *value = new Value();
+            value->append(QSharedPointer<VerbatimText>(new VerbatimText(visibleFilename)));
+            lineAdd(value);
+            delete value;
+            emit modified();
+        }
     }
 }
 
-void UrlListEdit::slotSaveLocally(QWidget *widget)
+void UrlListEdit::downloadAndSaveLocally(const QUrl &url)
 {
-    /// Determine FieldLineEdit widget
-    FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(widget);
-    /// Build Url from line edit's content
-    const QUrl url = QUrl::fromUserInput(fieldLineEdit->text());
-
     /// Only proceed if Url is valid and points to a remote location
-    if (url.isValid() && !urlIsLocal(url)) {
+    if (url.isValid() && !url.isLocalFile()) {
         /// Get filename from url (without any path/directory part)
         QString filename = url.fileName();
         /// Build QFileInfo from current BibTeX file if available
@@ -570,15 +555,9 @@ void UrlListEdit::downloadFinished(KJob *j) {
     unsetCursor();
 }
 
-void UrlListEdit::textChanged(QWidget *widget)
+void UrlListEdit::textChanged(QPushButton *buttonSaveLocally, FieldLineEdit *fieldLineEdit)
 {
-    /// Determine associated QPushButton "Save locally"
-    QPushButton *buttonSaveLocally = qobject_cast<QPushButton *>(widget);
-    if (buttonSaveLocally == nullptr) return; ///< should never happen!
-
-    /// Assume a FieldLineEdit was the sender of this signal
-    FieldLineEdit *fieldLineEdit = qobject_cast<FieldLineEdit *>(m_signalMapperFieldLineEditTextChanged->mapping(widget));
-    if (fieldLineEdit == nullptr) return; ///< should never happen!
+    if (buttonSaveLocally == nullptr || fieldLineEdit == nullptr) return; ///< should never happen!
 
     /// Create URL from new text to make some tests on it
     /// Only remote URLs are of interest, therefore no tests
@@ -596,9 +575,9 @@ void UrlListEdit::textChanged(QWidget *widget)
 
 QString UrlListEdit::askRelativeOrStaticFilename(QWidget *parent, const QString &absoluteFilename, const QUrl &baseUrl)
 {
-    QFileInfo baseUrlInfo = baseUrl.isEmpty() ? QFileInfo() : QFileInfo(baseUrl.path());
+    QFileInfo baseUrlInfo = baseUrl.isValid() ? QFileInfo(baseUrl.path()) : QFileInfo();
     QFileInfo filenameInfo(absoluteFilename);
-    if (!baseUrl.isEmpty() && (filenameInfo.absolutePath() == baseUrlInfo.absolutePath() || filenameInfo.absolutePath().startsWith(baseUrlInfo.absolutePath() + QDir::separator()))) {
+    if (baseUrl.isValid() && (filenameInfo.absolutePath() == baseUrlInfo.absolutePath() || filenameInfo.absolutePath().startsWith(baseUrlInfo.absolutePath() + QDir::separator()))) {
         // TODO cover level-up cases like "../../test.pdf"
         const QString relativePath = filenameInfo.absolutePath().mid(baseUrlInfo.absolutePath().length() + 1);
         const QString relativeFilename = relativePath + (relativePath.isEmpty() ? QString() : QString(QDir::separator())) + filenameInfo.fileName();
@@ -608,14 +587,6 @@ QString UrlListEdit::askRelativeOrStaticFilename(QWidget *parent, const QString
     return absoluteFilename;
 }
 
-bool UrlListEdit::urlIsLocal(const QUrl &url)
-{
-    // FIXME same function as in AssociateFiles; move to common code base?
-    const QString scheme = url.scheme();
-    /// Test various schemes such as "http", "https", "ftp", ...
-    return !scheme.startsWith(QStringLiteral("http")) && !scheme.startsWith(QStringLiteral("ftp")) && !scheme.startsWith(QStringLiteral("sftp")) && !scheme.startsWith(QStringLiteral("fish")) && !scheme.startsWith(QStringLiteral("webdav")) && scheme != QStringLiteral("smb");
-}
-
 FieldLineEdit *UrlListEdit::addFieldLineEdit()
 {
     /// Call original implementation to get an instance of a FieldLineEdit
@@ -629,10 +600,12 @@ FieldLineEdit *UrlListEdit::addFieldLineEdit()
     fieldLineEdit->appendWidget(buttonSaveLocally);
     /// Connect signals to react on button events
     /// or changes in the FieldLineEdit's text
-    m_signalMapperSaveLocallyButtonClicked->setMapping(buttonSaveLocally, fieldLineEdit);
-    m_signalMapperFieldLineEditTextChanged->setMapping(fieldLineEdit, buttonSaveLocally);
-    connect(buttonSaveLocally, &QPushButton::clicked, m_signalMapperSaveLocallyButtonClicked, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
-    connect(fieldLineEdit, &FieldLineEdit::textChanged, m_signalMapperFieldLineEditTextChanged, static_cast<void(QSignalMapper::*)()>(&QSignalMapper::map));
+    connect(buttonSaveLocally, &QPushButton::clicked, this, [this, fieldLineEdit]() {
+        downloadAndSaveLocally(QUrl::fromUserInput(fieldLineEdit->text()));
+    });
+    connect(fieldLineEdit, &FieldLineEdit::textChanged, this, [this, buttonSaveLocally, fieldLineEdit]() {
+        textChanged(buttonSaveLocally, fieldLineEdit);
+    });
 
     return fieldLineEdit;
 }
@@ -647,7 +620,7 @@ void UrlListEdit::setReadOnly(bool isReadOnly)
 const QString KeywordListEdit::keyGlobalKeywordList = QStringLiteral("globalKeywordList");
 
 KeywordListEdit::KeywordListEdit(QWidget *parent)
-        : FieldListEdit(KBibTeX::tfKeyword, KBibTeX::tfKeyword | KBibTeX::tfSource, parent), m_config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), m_configGroupName(QStringLiteral("Global Keywords"))
+        : FieldListEdit(KBibTeX::TypeFlag::Keyword, KBibTeX::TypeFlag::Keyword | KBibTeX::TypeFlag::Source, parent), m_config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), m_configGroupName(QStringLiteral("Global Keywords"))
 {
     m_buttonAddKeywordsFromList = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add Keywords from List"), this);
     m_buttonAddKeywordsFromList->setToolTip(i18n("Add keywords as selected from a pre-defined list of keywords"));
