@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2019 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,12 +18,12 @@
 #ifndef KBIBTEX_IO_FILEIMPORTER_H
 #define KBIBTEX_IO_FILEIMPORTER_H
 
+#include <QObject>
+
 #ifdef HAVE_KF5
 #include "kbibtexio_export.h"
 #endif // HAVE_KF5
 
-#include <QObject>
-
 class QIODevice;
 
 class File;
@@ -37,10 +37,10 @@ class KBIBTEXIO_EXPORT FileImporter : public QObject
     Q_OBJECT
 
 public:
-    enum MessageSeverity {
-        SeverityInfo, ///< Messages that are of informative type, such as additional comma for last key-value pair in BibTeX entry
-        SeverityWarning, ///< Messages that are of warning type, such as automatic corrections of BibTeX code without loss of information
-        SeverityError ///< Messages that are of error type, which point to issue where information may get lost, e.g. invalid syntax or incomplete data
+    enum class MessageSeverity {
+        Info, ///< Messages that are of informative type, such as additional comma for last key-value pair in BibTeX entry
+        Warning, ///< Messages that are of warning type, such as automatic corrections of BibTeX code without loss of information
+        Error ///< Messages that are of error type, which point to issue where information may get lost, e.g. invalid syntax or incomplete data
     };
 
     explicit FileImporter(QObject *parent);
@@ -64,7 +64,7 @@ public:
       * additional information is necessary or not.
       */
     virtual bool showImportDialog(QWidget *parent) {
-        Q_UNUSED(parent);
+        Q_UNUSED(parent)
         return true;
     }
 
