@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2020 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,17 +18,17 @@
 #ifndef KBIBTEX_IO_FILEIMPORTERBIBTEX_H
 #define KBIBTEX_IO_FILEIMPORTERBIBTEX_H
 
-#ifdef HAVE_KF5
-#include "kbibtexio_export.h"
-#endif // HAVE_KF5
-
 #include <QTextStream>
 #include <QSharedPointer>
 #include <QStringList>
 #include <QSet>
 
-#include "kbibtex.h"
-#include "fileimporter.h"
+#include <KBibTeX>
+#include <FileImporter>
+
+#ifdef HAVE_KF5
+#include "kbibtexio_export.h"
+#endif // HAVE_KF5
 
 class Element;
 class Comment;
@@ -49,9 +49,7 @@ class KBIBTEXIO_EXPORT FileImporterBibTeX : public FileImporter
     Q_OBJECT
 
 public:
-    static const char *defaultCodecName;
-
-    enum CommentHandling {IgnoreComments = 0, KeepComments = 1};
+    enum class CommentHandling {Ignore, Keep};
 
     /**
      * Creates an importer class to read a BibTeX file.
@@ -86,7 +84,7 @@ public:
      * @return A list of Person object containing the names
      * @see Person
      */
-    static QList<QSharedPointer<Person> > splitNames(const QString &text, const int line_number = 1, QObject *parent = nullptr);
+    static QList<QSharedPointer<Person> > splitNames(const QString &text);
 
     /**
      * Split a person's name into its parts and construct a Person object from them.
@@ -106,10 +104,10 @@ public slots:
     void cancel() override;
 
 private:
-    enum Token {
-        tAt = 1, tBracketOpen = 2, tBracketClose = 3, tAlphaNumText = 4, tComma = 5, tAssign = 6, tDoublecross = 7, tEOF = 0xffff, tUnknown = -1
+    enum class Token {
+        At = 1, BracketOpen = 2, BracketClose = 3, AlphaNumText = 4, Comma = 5, Assign = 6, Doublecross = 7, EndOfFile = 0xffff, Unknown = -1
     };
-    enum CommaContainment { ccNoComma = 0, ccContainsComma = 1 };
+    enum class CommaContainment { None, Contains };
 
     struct {
         int countCurlyBrackets, countQuotationMarks;
@@ -128,7 +126,7 @@ private:
 
     /// low-level character operations
     QChar m_prevChar, m_nextChar;
-    unsigned int m_lineNo;
+    int m_lineNo;
     QString m_prevLine, m_currentLine;
     bool readChar();
     bool readCharUntil(const QString &until);
@@ -165,6 +163,7 @@ private:
     static void contextSensitiveSplit(const QString &text, QStringList &segments);
 
     static QString bibtexAwareSimplify(const QString &text);
+    static QString rstrip(const QString &text);
 
     bool evaluateParameterComments(QTextStream *textStream, const QString &line, File *file);
     QString tokenidToString(Token token);
