@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2020 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,10 +23,13 @@
 #include <QTimer>
 #include <QStandardPaths>
 #include <QRegularExpression>
-#ifdef HAVE_QTWIDGETS
-#include <QListWidgetItem>
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
+#if QT_VERSION >= 0x050a00
+#include <QRandomGenerator>
+#endif // QT_VERSION
+#ifdef HAVE_QTWIDGETS
+#include <QListWidgetItem>
 #endif // HAVE_QTWIDGETS
 
 #ifdef HAVE_KF5
@@ -34,11 +37,18 @@
 #include <KMessageBox>
 #endif // HAVE_KF5
 
-#include "encoderlatex.h"
+#include <KBibTeX>
+#include <Encoder>
 #include "internalnetworkaccessmanager.h"
-#include "kbibtex.h"
+#include "onlinesearchabstract_p.h"
 #include "logging_networking.h"
 
+#if QT_VERSION >= 0x050a00
+#define randomGeneratorGlobalGenerate()  QRandomGenerator::global()->generate()
+#else // QT_VERSION
+#define randomGeneratorGlobalGenerate()  (qrand())
+#endif // QT_VERSION
+
 const QString OnlineSearchAbstract::queryKeyFreeText = QStringLiteral("free");
 const QString OnlineSearchAbstract::queryKeyTitle = QStringLiteral("title");
 const QString OnlineSearchAbstract::queryKeyAuthor = QStringLiteral("author");
@@ -55,9 +65,14 @@ const char *OnlineSearchAbstract::httpUnsafeChars = "%:/=+$?&\0";
 
 
 #ifdef HAVE_QTWIDGETS
-QStringList OnlineSearchQueryFormAbstract::authorLastNames(const Entry &entry)
+OnlineSearchAbstract::Form::Private::Private()
+        : config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))) {
+    /// nothing
+}
+
+QStringList OnlineSearchAbstract::Form::Private::authorLastNames(const Entry &entry)
 {
-    const EncoderLaTeX &encoder = EncoderLaTeX::instance();
+    const Encoder &encoder = Encoder::instance();
     const Value v = entry[Entry::ftAuthor];
     QStringList result;
     result.reserve(v.size());
@@ -69,6 +84,31 @@ QStringList OnlineSearchQueryFormAbstract::authorLastNames(const Entry &entry)
 
     return result;
 }
+
+QString OnlineSearchAbstract::Form::Private::guessFreeText(const Entry &entry)
+{
+    /// If there is a DOI value in this entry, use it as free text
+    static const QStringList doiKeys = {Entry::ftDOI, Entry::ftUrl};
+    for (const QString &doiKey : doiKeys)
+        if (!entry.value(doiKey).isEmpty()) {
+            const QString text = PlainTextValue::text(entry[doiKey]);
+            const QRegularExpressionMatch doiRegExpMatch = KBibTeX::doiRegExp.match(text);
+            if (doiRegExpMatch.hasMatch())
+                return doiRegExpMatch.captured(0);
+        }
+
+    /// If there is no free text yet (e.g. no DOI number), try to identify an arXiv eprint number
+    static const QStringList arxivKeys = {QStringLiteral("eprint"), Entry::ftNumber};
+    for (const QString &arxivKey : arxivKeys)
+        if (!entry.value(arxivKey).isEmpty()) {
+            const QString text = PlainTextValue::text(entry[arxivKey]);
+            const QRegularExpressionMatch arXivRegExpMatch = KBibTeX::arXivRegExpWithPrefix.match(text);
+            if (arXivRegExpMatch.hasMatch())
+                return arXivRegExpMatch.captured(1);
+        }
+
+    return QString();
+}
 #endif // HAVE_QTWIDGETS
 
 OnlineSearchAbstract::OnlineSearchAbstract(QObject *parent)
@@ -101,7 +141,7 @@ QIcon OnlineSearchAbstract::icon(QListWidgetItem *listWidgetItem)
     return QIcon::fromTheme(QStringLiteral("applications-internet"));
 }
 
-OnlineSearchQueryFormAbstract *OnlineSearchAbstract::customWidget(QWidget *) {
+OnlineSearchAbstract::Form *OnlineSearchAbstract::customWidget(QWidget *) {
     return nullptr;
 }
 
@@ -115,9 +155,10 @@ void OnlineSearchAbstract::startSearchFromForm()
 
 QString OnlineSearchAbstract::name()
 {
-    static const QRegularExpression invalidChars(QStringLiteral("[^-a-z0-9]"), QRegularExpression::CaseInsensitiveOption);
-    if (m_name.isEmpty())
+    if (m_name.isEmpty()) {
+        static const QRegularExpression invalidChars(QStringLiteral("[^-a-z0-9]"), QRegularExpression::CaseInsensitiveOption);
         m_name = label().remove(invalidChars);
+    }
     return m_name;
 }
 
@@ -331,7 +372,7 @@ QString OnlineSearchAbstract::decodeURL(QString rawText)
     return rawText;
 }
 
-QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlText, int startPos) const
+QMultiMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlText, int startPos) const
 {
     /// how to recognize HTML tags
     static const QString formTagEnd = QStringLiteral("</form>");
@@ -341,7 +382,7 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
     static const QString optionTagBegin = QStringLiteral("<option ");
 
     /// initialize result map
-    QMap<QString, QString> result;
+    QMultiMap<QString, QString> result;
 
     /// determined boundaries of (only) "form" tag
     int endPos = htmlText.indexOf(formTagEnd, startPos, Qt::CaseInsensitive);
@@ -361,17 +402,17 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
         if (!inputName.isEmpty()) {
             /// get value of input types
             if (inputType == QStringLiteral("hidden") || inputType == QStringLiteral("text") || inputType == QStringLiteral("submit"))
-                result[inputName] = inputValue;
+                result.replace(inputName, inputValue);
             else if (inputType == QStringLiteral("radio")) {
                 /// must be selected
                 if (htmlAttributeIsSelected(htmlText, p, QStringLiteral("checked"))) {
-                    result[inputName] = inputValue;
+                    result.replace(inputName, inputValue);
                 }
             } else if (inputType == QStringLiteral("checkbox")) {
                 /// must be checked
                 if (htmlAttributeIsSelected(htmlText, p, QStringLiteral("checked"))) {
                     /// multiple checkbox values with the same name are possible
-                    result.insertMulti(inputName, inputValue);
+                    result.insert(inputName, inputValue);
                 }
             }
         }
@@ -395,7 +436,7 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
             if (!selectName.isEmpty() && !optionValue.isEmpty()) {
                 /// if this "option" tag is "selected", store value
                 if (htmlAttributeIsSelected(htmlText, popt, QStringLiteral("selected"))) {
-                    result[selectName] = optionValue;
+                    result.replace(selectName, optionValue);
                 }
             }
 
@@ -439,14 +480,13 @@ void OnlineSearchAbstract::iconDownloadFinished()
         if (iconData[1] == 'P' && iconData[2] == 'N' && iconData[3] == 'G') {
             /// PNG files have string "PNG" at second to fourth byte
             extension = QStringLiteral(".png");
-        } else if (iconData[0] == (char)0x00 && iconData[1] == (char)0x00 && iconData[2] == (char)0x01 && iconData[3] == (char)0x00) {
+        } else if (iconData[0] == static_cast<char>(0x00) && iconData[1] == static_cast<char>(0x00) && iconData[2] == static_cast<char>(0x01) && iconData[3] == static_cast<char>(0x00)) {
             /// Microsoft Icon have first two bytes always 0x0000,
             /// third and fourth byte is 0x0001 (for .ico)
             extension = QStringLiteral(".ico");
         } else if (iconData[0] == '<') {
             /// HTML or XML code
-            const QString htmlCode = QString::fromUtf8(iconData);
-            qCDebug(LOG_KBIBTEX_NETWORKING) << "Received XML or HTML data from " << InternalNetworkAccessManager::removeApiKey(reply->url()).toDisplayString() << ": " << htmlCode.left(128);
+            qCDebug(LOG_KBIBTEX_NETWORKING) << "Received XML or HTML data from " << InternalNetworkAccessManager::removeApiKey(reply->url()).toDisplayString();
             return;
         } else {
             qCWarning(LOG_KBIBTEX_NETWORKING) << "Favicon is of unknown format: " << InternalNetworkAccessManager::removeApiKey(reply->url()).toDisplayString();
@@ -460,8 +500,13 @@ void OnlineSearchAbstract::iconDownloadFinished()
             iconFile.close();
 
             QListWidgetItem *listWidgetItem = m_iconReplyToListWidgetItem.value(reply, nullptr);
-            if (listWidgetItem != nullptr)
+            if (listWidgetItem != nullptr) {
+                /// Disable signals while updating the widget and its items
+                blockSignals(true);
                 listWidgetItem->setIcon(QIcon(filename));
+                /// Re-enable signals after updating the widget and its items
+                blockSignals(false);
+            }
         } else {
             qCWarning(LOG_KBIBTEX_NETWORKING) << "Could not save icon data from URL" << InternalNetworkAccessManager::removeApiKey(reply->url()).toDisplayString() << "to file" << filename;
             return;
@@ -486,12 +531,9 @@ void OnlineSearchAbstract::dumpToFile(const QString &filename, const QString &te
 void OnlineSearchAbstract::delayedStoppedSearch(int returnCode)
 {
     m_delayedStoppedSearchReturnCode = returnCode;
-    QTimer::singleShot(500, this, &OnlineSearchAbstract::delayedStoppedSearchTimer);
-}
-
-void OnlineSearchAbstract::delayedStoppedSearchTimer()
-{
-    stopSearch(m_delayedStoppedSearchReturnCode);
+    QTimer::singleShot(500, this, [this]() {
+        stopSearch(m_delayedStoppedSearchReturnCode);
+    });
 }
 
 void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
@@ -500,7 +542,7 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
 
     /// Sometimes, there is no identifier, so set a random one
     if (entry->id().isEmpty())
-        entry->setId(QString(QStringLiteral("entry-%1")).arg(QString::number(qrand(), 36)));
+        entry->setId(QString(QStringLiteral("entry-%1")).arg(QString::number(randomGeneratorGlobalGenerate(), 36)));
     /// Missing entry type? Set it to 'misc'
     if (entry->type().isEmpty())
         entry->setType(Entry::etMisc);
@@ -577,7 +619,6 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
         /// If URL looks like a DOI, remove URL and add a DOI field
         QSet<QString> doiSet; ///< using a QSet here to keep only unique DOIs
         Value v = entry->value(Entry::ftUrl);
-        doiSet.reserve(v.length());
         bool gotChanged = false;
         for (Value::Iterator it = v.begin(); it != v.end();) {
             const QString viText = PlainTextValue::text(*it);
@@ -651,3 +692,16 @@ void OnlineSearchAbstract::refreshBusyProperty() {
         emit busyChanged();
     }
 }
+
+#ifdef HAVE_QTWIDGETS
+OnlineSearchAbstract::Form::Form(QWidget *parent)
+        : QWidget(parent), d(new OnlineSearchAbstract::Form::Private())
+{
+    /// nothing
+}
+
+OnlineSearchAbstract::Form::~Form()
+{
+    delete d;
+}
+#endif // HAVE_QTWIDGETS
