@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2022 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2020 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -35,9 +35,9 @@
 #define i18n(text) QObject::tr(text)
 #endif // HAVE_KF5
 
-#include "file.h"
-#include "entry.h"
-#include "fileimporterbibtex.h"
+#include <File>
+#include <Entry>
+#include <FileImporterBibTeX>
 #include "internalnetworkaccessmanager.h"
 #include "logging_networking.h"
 
@@ -64,8 +64,12 @@ public:
         /// to KBibTeX's authors along with information which ACM records were search
         /// for or actually found.
         static const QSet<QString> evaluatedOrIgnoredValues{QStringLiteral("id"), QStringLiteral("call-number"), QStringLiteral("DOI"), QStringLiteral("ISBN"), QStringLiteral("ISSN"), QStringLiteral("URL"), QStringLiteral("type"), QStringLiteral("author"), QStringLiteral("issued"), QStringLiteral("original-date"), QStringLiteral("number-of-pages"), QStringLiteral("page"), QStringLiteral("number"), QStringLiteral("issue"), QStringLiteral("volume"), QStringLiteral("publisher"), QStringLiteral("publisher-place"), QStringLiteral("title"), QStringLiteral("collection-title"), QStringLiteral("container-title"), QStringLiteral("event-place"), QStringLiteral("keyword"), QStringLiteral("accessed")};
+#if QT_VERSION >= 0x050e00
+        const QSet<QString> objectsValues = QSet<QString>(object.keys().begin(), object.keys().end());
+#else // QT_VERSION < 0x050e00
         QSet<QString> objectsValues;
         for (const QString &objectValue : object.keys()) objectsValues.insert(objectValue);
+#endif // QT_VERSION
         const QSet<QString> unusedValues = objectsValues - evaluatedOrIgnoredValues;
         for (const QString &unusedValue : unusedValues) {
             const QString valueString = object.value(unusedValue).toString();
@@ -299,7 +303,12 @@ void OnlineSearchAcmPortal::startSearch(const QMap<QString, QString> &query, int
 
 QString OnlineSearchAcmPortal::label() const
 {
+#ifdef HAVE_KF5
     return i18n("ACM Digital Library");
+#else // HAVE_KF5
+    //= onlinesearch-acmdigitallibrary-label
+    return QObject::tr("ACM Digital Library");
+#endif // HAVE_KF5
 }
 
 QString OnlineSearchAcmPortal::favIconUrl() const
@@ -361,15 +370,16 @@ void OnlineSearchAcmPortal::doneFetchingSearchPage()
         const QString htmlSource = QString::fromUtf8(reply->readAll().constData());
 
         QVector<QString> dois;
-        static const QRegularExpression citationUrlRegExp(QStringLiteral("/doi(/abs)?/(") + KBibTeX::doiRegExp.pattern() + QStringLiteral(")"), QRegularExpression::CaseInsensitiveOption);
+        static const QRegularExpression citationUrlRegExp(QStringLiteral("/doi/abs/(") + KBibTeX::doiRegExp.pattern() + QStringLiteral(")"), QRegularExpression::CaseInsensitiveOption);
         QRegularExpressionMatchIterator citationUrlRegExpMatchIt = citationUrlRegExp.globalMatch(htmlSource);
         while (citationUrlRegExpMatchIt.hasNext()) {
             const QRegularExpressionMatch citationUrlRegExpMatch = citationUrlRegExpMatchIt.next();
-            dois.append(citationUrlRegExpMatch.captured(2));
+            dois.append(citationUrlRegExpMatch.captured(1));
             if (d->currentSearchPosition + dois.length() > d->numExpectedResults)
                 break;
         }
 
+        bool stillBusy = false;
         if (d->currentSearchPosition + OnlineSearchAcmPortalPrivate::requestedResultsPerPage < d->numExpectedResults) {
             ++numSteps;
             d->currentSearchPosition += OnlineSearchAcmPortalPrivate::requestedResultsPerPage;
@@ -383,6 +393,7 @@ void OnlineSearchAcmPortal::doneFetchingSearchPage()
             QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
             InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
             connect(newReply, &QNetworkReply::finished, this, &OnlineSearchAcmPortal::doneFetchingSearchPage);
+            stillBusy = true;
         }
         if (!dois.isEmpty()) {
             QNetworkRequest request(QUrl(QStringLiteral("https://dl.acm.org/action/exportCiteProcCitation")));
@@ -400,7 +411,10 @@ void OnlineSearchAcmPortal::doneFetchingSearchPage()
             QNetworkReply *newReply = InternalNetworkAccessManager::instance().post(request, postBody);
             InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
             connect(newReply, &QNetworkReply::finished, this, &OnlineSearchAcmPortal::doneFetchingJSON);
+            stillBusy = true;
         }
+        if (!stillBusy)
+            stopSearch(resultNoError);
     }
 
     refreshBusyProperty();
