@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2019 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,11 +21,11 @@
 #include <QMap>
 #include <QLabel>
 #include <QListWidget>
+#include <QLineEdit>
 #include <QSpinBox>
 #include <QStackedWidget>
 #include <QTabWidget>
 #include <QProgressBar>
-#include <QMimeDatabase>
 #include <QMimeType>
 #include <QTimer>
 #include <QSet>
@@ -33,48 +33,49 @@
 #include <QScrollArea>
 #include <QIcon>
 #include <QPushButton>
-#include <QDebug>
 
-#include <KLineEdit>
+#include <kio_version.h>
 #include <KLocalizedString>
+#if KIO_VERSION >= 0x054700 // >= 5.71.0
+#include <KIO/OpenUrlJob>
+#include <KIO/JobUiDelegate>
+#else // < 5.71.0
 #include <KRun>
+#endif // KIO_VERSION >= 0x054700
 #include <KMessageBox>
 #include <KParts/Part>
 #include <KParts/ReadOnlyPart>
 #include <KConfigGroup>
 #include <KSharedConfig>
-#include <kio_version.h>
 
-#include "element.h"
-#include "file.h"
-#include "comment.h"
-#include "fileexporterbibtex.h"
-#include "onlinesearchabstract.h"
-#include "onlinesearchgeneral.h"
-#include "onlinesearchbibsonomy.h"
-#include "onlinesearchgooglescholar.h"
-#include "onlinesearchpubmed.h"
-#include "onlinesearchieeexplore.h"
-#include "onlinesearchacmportal.h"
-#include "onlinesearchsciencedirect.h"
-#include "onlinesearchspringerlink.h"
-#include "onlinesearcharxiv.h"
-#ifdef HAVE_WEBENGINEWIDGETS
-#include "onlinesearchjstor.h"
-#endif // HAVE_WEBENGINEWIDGETS
-#include "onlinesearchmathscinet.h"
-#include "onlinesearchmrlookup.h"
-#include "onlinesearchinspirehep.h"
-#include "onlinesearchcernds.h"
-#include "onlinesearchingentaconnect.h"
-#include "onlinesearchsoanasaads.h"
-#include "onlinesearchisbndb.h"
-#include "onlinesearchideasrepec.h"
-#include "onlinesearchdoi.h"
-#include "onlinesearchbiorxiv.h"
+#include <Element>
+#include <File>
+#include <Comment>
+#include <FileExporterBibTeX>
+#include <onlinesearch/OnlineSearchAbstract>
+#include <onlinesearch/OnlineSearchGeneral>
+#include <onlinesearch/OnlineSearchBibsonomy>
+#include <onlinesearch/onlinesearchgooglescholar.h>
+#include <onlinesearch/OnlineSearchPubMed>
+#include <onlinesearch/OnlineSearchIEEEXplore>
+#include <onlinesearch/OnlineSearchAcmPortal>
+#include <onlinesearch/OnlineSearchScienceDirect>
+#include <onlinesearch/OnlineSearchSpringerLink>
+#include <onlinesearch/OnlineSearchArXiv>
+#include <onlinesearch/OnlineSearchJStor>
+#include <onlinesearch/OnlineSearchMathSciNet>
+#include <onlinesearch/OnlineSearchMRLookup>
+#include <onlinesearch/OnlineSearchInspireHep>
+#include <onlinesearch/OnlineSearchCERNDS>
+#include <onlinesearch/OnlineSearchIngentaConnect>
+#include <onlinesearch/OnlineSearchSOANASAADS>
+#include <onlinesearch/OnlineSearchIDEASRePEc>
+#include <onlinesearch/OnlineSearchDOI>
+#include <onlinesearch/OnlineSearchBioRxiv>
+#include <onlinesearch/OnlineSearchSemanticScholar>
+#include <file/FileView>
+#include <models/FileModel>
 #include "openfileinfo.h"
-#include "fileview.h"
-#include "models/filemodel.h"
 #include "searchresults.h"
 #include "logging_program.h"
 
@@ -97,12 +98,12 @@ public:
     QSet<OnlineSearchAbstract *> runningSearches;
     QPushButton *searchButton;
     QPushButton *useEntryButton;
-    OnlineSearchQueryFormGeneral *generalQueryTermsForm;
+    OnlineSearchGeneral::Form *generalQueryTermsForm;
     QTabWidget *tabWidget;
     QSharedPointer<const Entry> currentEntry;
     QProgressBar *progressBar;
     QMap<OnlineSearchAbstract *, int> progressMap;
-    QMap<OnlineSearchQueryFormAbstract *, QScrollArea *> formToScrollArea;
+    QMap<OnlineSearchAbstract::Form *, QScrollArea *> formToScrollArea;
 
     enum SearchFormPrivateRole {
         /// Homepage of a search engine
@@ -119,12 +120,12 @@ public:
         createGUI();
     }
 
-    OnlineSearchQueryFormAbstract *currentQueryForm() {
+    OnlineSearchAbstract::Form *currentQueryForm() {
         QScrollArea *area = qobject_cast<QScrollArea *>(queryTermsStack->currentWidget());
         return formToScrollArea.key(area, nullptr);
     }
 
-    QScrollArea *wrapInScrollArea(OnlineSearchQueryFormAbstract *form, QWidget *parent) {
+    QScrollArea *wrapInScrollArea(OnlineSearchAbstract::Form *form, QWidget *parent) {
         QScrollArea *scrollArea = new QScrollArea(parent);
         form->setParent(scrollArea);
         scrollArea->setWidget(form);
@@ -144,7 +145,9 @@ public:
         whichEnginesLabel->setWordWrap(true);
         vLayout->addWidget(whichEnginesLabel);
         vLayout->setStretchFactor(whichEnginesLabel, 0);
-        connect(whichEnginesLabel, &QLabel::linkActivated, p, &SearchForm::switchToEngines);
+        connect(whichEnginesLabel, &QLabel::linkActivated, p, [this]() {
+            switchToEngines();
+        });
 
         vLayout->addSpacing(8);
 
@@ -158,8 +161,8 @@ public:
         return container;
     }
 
-    OnlineSearchQueryFormAbstract *createGeneralQueryTermsForm(QWidget *parent = nullptr) {
-        generalQueryTermsForm = new OnlineSearchQueryFormGeneral(parent);
+    OnlineSearchGeneral::Form *createGeneralQueryTermsForm(QWidget *parent = nullptr) {
+        generalQueryTermsForm = new OnlineSearchGeneral::Form(parent);
         return generalQueryTermsForm;
     }
 
@@ -172,11 +175,15 @@ public:
         enginesList = new QListWidget(listContainer);
         layout->addWidget(enginesList, 0, 0, 1, 1);
         connect(enginesList, &QListWidget::itemChanged, p, &SearchForm::itemCheckChanged);
-        connect(enginesList, &QListWidget::currentItemChanged, p, &SearchForm::enginesListCurrentChanged);
+        connect(enginesList, &QListWidget::currentItemChanged, p, [this](QListWidgetItem * current, QListWidgetItem *) {
+            enginesListCurrentChanged(current);
+        });
         enginesList->setSelectionMode(QAbstractItemView::NoSelection);
 
         actionOpenHomepage = new QAction(QIcon::fromTheme(QStringLiteral("internet-web-browser")), i18n("Go to Homepage"), p);
-        connect(actionOpenHomepage, &QAction::triggered, p, &SearchForm::openHomepage);
+        connect(actionOpenHomepage, &QAction::triggered, p, [this]() {
+            openHomepage();
+        });
         enginesList->addAction(actionOpenHomepage);
         enginesList->setContextMenuPolicy(Qt::ActionsContextMenu);
 
@@ -202,7 +209,9 @@ public:
         QWidget *listContainer = createEnginesGUI(tabWidget);
         tabWidget->addTab(listContainer, QIcon::fromTheme(QStringLiteral("applications-engineering")), i18n("Engines"));
 
-        connect(tabWidget, &QTabWidget::currentChanged, p, &SearchForm::tabSwitched);
+        connect(tabWidget, &QTabWidget::currentChanged, p, [this]() {
+            updateGUI();
+        });
 
         useEntryButton = new QPushButton(QIcon::fromTheme(QStringLiteral("go-up")), i18n("Use Entry"), p);
         layout->addWidget(useEntryButton, 1, 0, 1, 1);
@@ -216,7 +225,7 @@ public:
 
         searchButton = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-find")), i18n("Search"), p);
         layout->addWidget(searchButton, 1, 2, 1, 1);
-        connect(generalQueryTermsForm, &OnlineSearchQueryFormGeneral::returnPressed, searchButton, &QPushButton::click);
+        connect(generalQueryTermsForm, &OnlineSearchAbstract::Form::returnPressed, searchButton, &QPushButton::click);
 
         updateGUI();
     }
@@ -231,9 +240,7 @@ public:
         addEngine(new OnlineSearchGoogleScholar(p));
         addEngine(new OnlineSearchIEEEXplore(p));
         addEngine(new OnlineSearchIngentaConnect(p));
-#ifdef HAVE_WEBENGINEWIDGETS
         addEngine(new OnlineSearchJStor(p));
-#endif // HAVE_WEBENGINEWIDGETS
         addEngine(new OnlineSearchMathSciNet(p));
         addEngine(new OnlineSearchMRLookup(p));
         addEngine(new OnlineSearchInspireHep(p));
@@ -245,6 +252,7 @@ public:
         /// addEngine(new OnlineSearchIsbnDB(p)); /// disabled as provider switched to a paid model on 2017-12-26
         addEngine(new OnlineSearchIDEASRePEc(p));
         addEngine(new OnlineSearchDOI(p));
+        addEngine(new OnlineSearchSemanticScholar(p));
 
         p->itemCheckChanged(nullptr);
         updateGUI();
@@ -253,25 +261,34 @@ public:
     void addEngine(OnlineSearchAbstract *engine) {
         KConfigGroup configGroup(config, configGroupName);
 
+        /// Disable signals while updating the widget and its items
+        enginesList->blockSignals(true);
+
         QListWidgetItem *item = new QListWidgetItem(engine->label(), enginesList);
-        item->setCheckState(configGroup.readEntry(engine->name(), false) ? Qt::Checked : Qt::Unchecked);
+        static const QSet<QString> enginesEnabledByDefault {QStringLiteral("GoogleScholar"), QStringLiteral("Bibsonomy")};
+        item->setCheckState(configGroup.readEntry(engine->name(), enginesEnabledByDefault.contains(engine->name())) ? Qt::Checked : Qt::Unchecked);
         item->setIcon(engine->icon(item));
         item->setToolTip(engine->label());
         item->setData(HomepageRole, engine->homepage());
         item->setData(NameRole, engine->name());
 
-        OnlineSearchQueryFormAbstract *widget = engine->customWidget(queryTermsStack);
-        item->setData(WidgetRole, QVariant::fromValue<OnlineSearchQueryFormAbstract *>(widget));
+        OnlineSearchAbstract::Form *widget = engine->customWidget(queryTermsStack);
+        item->setData(WidgetRole, QVariant::fromValue<OnlineSearchAbstract::Form *>(widget));
         if (widget != nullptr) {
-            connect(widget, &OnlineSearchQueryFormAbstract::returnPressed, searchButton, &QPushButton::click);
+            connect(widget, &OnlineSearchAbstract::Form::returnPressed, searchButton, &QPushButton::click);
             QScrollArea *scrollArea = wrapInScrollArea(widget, queryTermsStack);
             queryTermsStack->addWidget(scrollArea);
         }
 
         itemToOnlineSearch.insert(item, engine);
-        connect(engine, &OnlineSearchAbstract::foundEntry, p, &SearchForm::foundEntry);
+        connect(engine, &OnlineSearchAbstract::foundEntry, p, [this](QSharedPointer<Entry> entry) {
+            sr->insertElement(entry);
+        });
         connect(engine, &OnlineSearchAbstract::stoppedSearch, p, &SearchForm::stoppedSearch);
         connect(engine, &OnlineSearchAbstract::progress, p, &SearchForm::updateProgress);
+
+        /// Re-enable signals after updating the widget and its items
+        enginesList->blockSignals(false);
     }
 
     void switchToSearch() {
@@ -321,9 +338,9 @@ public:
         default: whichEnginesLabel->setText(i18n("Search engines <b>%1</b>, <b>%2</b>, and more are selected (<a href=\"changeEngine\">change</a>).", checkedEngines.first(), checkedEngines.at(1))); break;
         }
 
-        OnlineSearchQueryFormAbstract *currentQueryWidget = nullptr;
+        OnlineSearchAbstract::Form *currentQueryWidget = nullptr;
         if (cursor != nullptr && checkedEngines.size() == 1)
-            currentQueryWidget = cursor->data(WidgetRole).value<OnlineSearchQueryFormAbstract *>();
+            currentQueryWidget = cursor->data(WidgetRole).value<OnlineSearchAbstract::Form *>();
         if (currentQueryWidget == nullptr)
             currentQueryWidget = generalQueryTermsForm;
         QScrollArea *area = formToScrollArea.value(currentQueryWidget, nullptr);
@@ -342,11 +359,13 @@ public:
             QMimeType mimeType = FileInfo::mimeTypeForUrl(url);
             const QString mimeTypeName = mimeType.name();
             /// Ask KDE subsystem to open url in viewer matching mime type
-#if KIO_VERSION < 0x051f00 // < 5.31.0
-            KRun::runUrl(url, mimeTypeName, p, false, false);
-#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+#if KIO_VERSION < 0x054700 // < 5.71.0
             KRun::runUrl(url, mimeTypeName, p, KRun::RunFlags());
-#endif // KIO_VERSION < 0x051f00
+#else // KIO_VERSION < 0x054700 // >= 5.71.0
+            KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url, mimeTypeName);
+            job->setUiDelegate(new KIO::JobUiDelegate());
+            job->start();
+#endif // KIO_VERSION < 0x054700
         }
     }
 
@@ -367,25 +386,15 @@ SearchForm::~SearchForm()
     delete d;
 }
 
-void SearchForm::updatedConfiguration()
-{
-    d->loadEngines();
-}
-
 void SearchForm::setElement(QSharedPointer<Element> element, const File *)
 {
     d->currentEntry = element.dynamicCast<const Entry>();
     d->useEntryButton->setEnabled(!d->currentEntry.isNull() && d->tabWidget->currentIndex() == 0);
 }
 
-void SearchForm::switchToEngines()
-{
-    d->switchToEngines();
-}
-
 void SearchForm::startSearch()
 {
-    OnlineSearchQueryFormAbstract *currentForm = d->currentQueryForm();
+    OnlineSearchAbstract::Form *currentForm = d->currentQueryForm();
     if (!currentForm->readyToStart()) {
         KMessageBox::sorry(this, i18n("Could not start searching the Internet:\nThe search terms are not complete or invalid."), i18n("Searching the Internet"));
         return;
@@ -429,11 +438,6 @@ void SearchForm::startSearch()
     d->switchToCancel();
 }
 
-void SearchForm::foundEntry(QSharedPointer<Entry> entry)
-{
-    d->sr->insertElement(entry);
-}
-
 void SearchForm::stoppedSearch(int)
 {
     OnlineSearchAbstract *engine = static_cast<OnlineSearchAbstract *>(sender());
@@ -457,12 +461,6 @@ void SearchForm::stoppedSearch(int)
     }
 }
 
-void SearchForm::tabSwitched(int newTab)
-{
-    Q_UNUSED(newTab);
-    d->updateGUI();
-}
-
 void SearchForm::itemCheckChanged(QListWidgetItem *item)
 {
     int numCheckedEngines = 0;
@@ -480,16 +478,6 @@ void SearchForm::itemCheckChanged(QListWidgetItem *item)
     }
 }
 
-void SearchForm::openHomepage()
-{
-    d->openHomepage();
-}
-
-void SearchForm::enginesListCurrentChanged(QListWidgetItem *current, QListWidgetItem *)
-{
-    d->enginesListCurrentChanged(current);
-}
-
 void SearchForm::copyFromEntry()
 {
     Q_ASSERT_X(!d->currentEntry.isNull(), "SearchForm::copyFromEntry", "d->currentEntry is NULL");
