@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2019 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -36,25 +36,24 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 
-#include "kbibtex.h"
-#include "preferences/kbibtexpreferencesdialog.h"
-#include "valuelist.h"
-#ifdef HAVE_ZOTERO
-#include "zoterobrowser.h"
-#endif // HAVE_ZOTERO
-#include "statistics.h"
+#include <KBibTeX>
+#include <preferences/KBibTeXPreferencesDialog>
+#include <file/FileView>
+#include <XSLTransform>
+#include <BibliographyService>
+#include <BibUtils>
+#include "docklets/referencepreview.h"
+#include "docklets/documentpreview.h"
+#include "docklets/searchform.h"
+#include "docklets/searchresults.h"
+#include "docklets/elementform.h"
+#include "docklets/documentpreview.h"
+#include "docklets/statistics.h"
+#include "docklets/filesettings.h"
+#include "docklets/valuelist.h"
+#include "docklets/zoterobrowser.h"
 #include "documentlist.h"
 #include "mdiwidget.h"
-#include "referencepreview.h"
-#include "documentpreview.h"
-#include "searchform.h"
-#include "searchresults.h"
-#include "elementform.h"
-#include "fileview.h"
-#include "filesettings.h"
-#include "xsltransform.h"
-#include "bibliographyservice.h"
-#include "bibutils.h"
 
 class KBibTeXMainWindow::KBibTeXMainWindowPrivate
 {
@@ -67,9 +66,7 @@ public:
     QDockWidget *dockReferencePreview;
     QDockWidget *dockDocumentPreview;
     QDockWidget *dockValueList;
-#ifdef HAVE_ZOTERO
     QDockWidget *dockZotero;
-#endif // HAVE_ZOTERO
     QDockWidget *dockStatistics;
     QDockWidget *dockSearchForm;
     QDockWidget *dockSearchResults;
@@ -81,9 +78,7 @@ public:
     DocumentPreview *documentPreview;
     FileSettings *fileSettings;
     ValueList *valueList;
-#ifdef HAVE_ZOTERO
     ZoteroBrowser *zotero;
-#endif // HAVE_ZOTERO
     Statistics *statistics;
     SearchForm *searchForm;
     SearchResults *searchResults;
@@ -166,7 +161,6 @@ public:
         dockSearchForm->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
         showPanelsMenu->addAction(dockSearchForm->toggleViewAction());
 
-#ifdef HAVE_ZOTERO
         dockZotero = new QDockWidget(i18n("Zotero"), p);
         dockZotero->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
         p->addDockWidget(Qt::LeftDockWidgetArea, dockZotero);
@@ -177,7 +171,6 @@ public:
         dockZotero->setObjectName(QStringLiteral("dockZotero"));
         dockZotero->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
         showPanelsMenu->addAction(dockZotero->toggleViewAction());
-#endif // HAVE_ZOTERO
 
         dockReferencePreview = new QDockWidget(i18n("Reference Preview"), p);
         dockReferencePreview->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
@@ -219,9 +212,7 @@ public:
         showPanelsMenu->addAction(dockFileSettings->toggleViewAction());
 
         p->tabifyDockWidget(dockFileSettings, dockSearchForm);
-#ifdef HAVE_ZOTERO
         p->tabifyDockWidget(dockZotero, dockSearchForm);
-#endif // HAVE_ZOTERO
         p->tabifyDockWidget(dockValueList, dockStatistics);
         p->tabifyDockWidget(dockStatistics, dockFileSettings);
         p->tabifyDockWidget(dockSearchForm, dockReferencePreview);
@@ -232,23 +223,19 @@ public:
         action = p->actionCollection()->addAction(KStandardAction::Open);
         connect(action, &QAction::triggered, p, &KBibTeXMainWindow::openDocumentDialog);
         actionClose = p->actionCollection()->addAction(KStandardAction::Close);
-        connect(actionClose, &QAction::triggered, p, &KBibTeXMainWindow::closeDocument);
+        connect(actionClose, &QAction::triggered, p, []() {
+            OpenFileInfoManager::instance().close(OpenFileInfoManager::instance().currentFile());
+        });
         actionClose->setEnabled(false);
         action = p->actionCollection()->addAction(KStandardAction::Quit);
         connect(action, &QAction::triggered, p, &KBibTeXMainWindow::queryCloseAll);
         action = p->actionCollection()->addAction(KStandardAction::Preferences);
         connect(action, &QAction::triggered, p, &KBibTeXMainWindow::showPreferences);
     }
-
-    ~KBibTeXMainWindowPrivate() {
-        elementForm->deleteLater();
-        delete mdiWidget;
-        // TODO other deletes
-    }
 };
 
 KBibTeXMainWindow::KBibTeXMainWindow(QWidget *parent)
-        : KParts::MainWindow(parent, (Qt::WindowFlags)KDE_DEFAULT_WINDOWFLAGS), d(new KBibTeXMainWindowPrivate(this))
+        : KParts::MainWindow(parent), d(new KBibTeXMainWindowPrivate(this))
 {
     setObjectName(QStringLiteral("KBibTeXShell"));
 
@@ -265,7 +252,7 @@ KBibTeXMainWindow::KBibTeXMainWindow(QWidget *parent)
     connect(&OpenFileInfoManager::instance(), &OpenFileInfoManager::flagsChanged, this, &KBibTeXMainWindow::documentListsChanged);
     connect(d->mdiWidget, &MDIWidget::setCaption, this, static_cast<void(KMainWindow::*)(const QString &)>(&KMainWindow::setCaption)); ///< actually: KMainWindow::setCaption
 
-    documentListsChanged(OpenFileInfo::RecentlyUsed); /// force initialization of menu of recently used files
+    documentListsChanged(OpenFileInfo::StatusFlag::RecentlyUsed); /// force initialization of menu of recently used files
 
     setupControllers();
     setupGUI(KXmlGuiWindow::Create | KXmlGuiWindow::Save | KXmlGuiWindow::Keys | KXmlGuiWindow::ToolBar);
@@ -348,7 +335,7 @@ void KBibTeXMainWindow::openDocumentDialog()
 
     delete dlg;
 
-    if (!url.isEmpty())
+    if (url.isValid())
         openDocument(url);
 }
 
@@ -358,11 +345,6 @@ void KBibTeXMainWindow::openDocument(const QUrl &url)
     OpenFileInfoManager::instance().setCurrentFile(openFileInfo);
 }
 
-void KBibTeXMainWindow::closeDocument()
-{
-    OpenFileInfoManager::instance().close(OpenFileInfoManager::instance().currentFile());
-}
-
 void KBibTeXMainWindow::closeEvent(QCloseEvent *event)
 {
     KMainWindow::closeEvent(event);
@@ -431,8 +413,8 @@ void KBibTeXMainWindow::showSearchResults()
 
 void KBibTeXMainWindow::documentListsChanged(OpenFileInfo::StatusFlags statusFlags)
 {
-    if (statusFlags.testFlag(OpenFileInfo::RecentlyUsed)) {
-        const OpenFileInfoManager::OpenFileInfoList list = OpenFileInfoManager::instance().filteredItems(OpenFileInfo::RecentlyUsed);
+    if (statusFlags.testFlag(OpenFileInfo::StatusFlag::RecentlyUsed)) {
+        const OpenFileInfoManager::OpenFileInfoList list = OpenFileInfoManager::instance().filteredItems(OpenFileInfo::StatusFlag::RecentlyUsed);
         d->actionMenuRecentFilesMenu->clear();
         for (OpenFileInfo *cur : list) {
             /// Fixing bug 19511: too long filenames make menu too large,
